/*

* GTNetS provides a portable C++ class library for network simulations
*
* Copyright (C) 2003 George F. Riley
*
* No guarantees or warranties or anything are provided or implied in any way
* whatsoever.  Use this program at your own risk.  Permission to use this
* program for any purpose is given, as long as the copyright is kept intact.
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

* Permission is hereby granted for any non-commercial use of this code
* $Id: aodv-queue.cc,v 1.1.1.1 2006/12/13 13:09:35 dnguyen Exp $

*/
//
// Georgia Tech Network Simulator - AODV queue class
// George F. Riley.  Georgia Tech, Spring 2002
//

#include <assert.h>
#include "aodv-queue.h"
#include "simulator.h"
#include "ipv4.h" 	//for IPV4Header

#define CURRENT_TIME   Simulator::Now()

using namespace std;
// Packet Queue used by AODV. 

AODVQueue::AODVQueue() {
	nLimit = AODV_RTQ_MAX_LEN;
	timeout = AODV_RTQ_TIMEOUT;
}

void
AODVQueue::enque(Packet *pPacket) {
	int nLen = pkts.size();

 	purge(); // Purge any packets that have timed out.
 	pPacket->expire = CURRENT_TIME + timeout;
	if (nLen == nLimit) {
		Packet *p0 = pkts.front();	// get the 1st in Q
		pkts.pop_front();	// remove the 1st in Q afterwards
		assert(p0);
		if(p0->expire > CURRENT_TIME)
			//TODO: trace this one,tells DROP because of QFULL or TIMEOUT
			DEBUG0((cout << "DROP because of QFULL" <<endl));
		else 
			DEBUG0((cout << "DROP because of TIMEOUT" <<endl));

		//any case, free the packet.
		delete p0;
	 }
	pkts.push_back(pPacket);      
}
                

Packet*
AODVQueue::deque() {
	Packet *p;

	purge(); 
	p = pkts.front(); //get the 1st in Q
	pkts.pop_front(); //remove it from the Q
 	return p;
}

// Note: this queue buffers those packet which has no route
//       therefore it works at IP layer, this is why I could
//       PEEK ip header so easily.
Packet*
AODVQueue::deque(IPAddr_t dst) {
	Packet *p;

	purge();
	for(PktList_t::iterator i=pkts.begin(); i!=pkts.end();i++ ) {
		p = *i;
		IPV4Header *ipHdr = (IPV4Header*)p->PeekPDU();
		if((ipHdr->dst.ip) == dst){ 
			pkts.erase(i); //found one, then return, thus no need i--
			return p;
		}
	}

	return nil;
}

char 
AODVQueue::find(IPAddr_t dst) {
	Packet *p;
	
	for(PktList_t::iterator i=pkts.begin(); i!=pkts.end();i++ ) {
		p = *i;
		IPV4Header *ipHdr = (IPV4Header*)p->PeekPDU();
		if((ipHdr->dst).ip == dst) 
			return 1;
	}
   return 0;

}

void
AODVQueue::purge() {
	
	for(PktList_t::iterator i=pkts.begin(); i!=pkts.end();i++ ) {
		if((*i)->expire < CURRENT_TIME) {
			pkts.erase(i);
			i--;
		}
	}

}

