/*

* GTNetS provides a portable C++ class library for network simulations
*
* Copyright (C) 2003 George F. Riley
*
* No guarantees or warranties or anything are provided or implied in any way
* whatsoever.  Use this program at your own risk.  Permission to use this
* program for any purpose is given, as long as the copyright is kept intact.
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

* Permission is hereby granted for any non-commercial use of this code

$Id: aodv-queue.h,v 1.1.1.1 2006/12/13 13:09:35 dnguyen Exp $

*/
//
// Georgia Tech Network Simulator - AODV queue class
// George F. Riley.  Georgia Tech, Spring 2002
//


#ifndef __aodv_queue_h__
#define __aodv_queue_h_

#include <list>
#include "common-defs.h"
#include "packet.h"

// The maximum number of packets that we allow a routing
//  protocol to buffer.
#define AODV_RTQ_MAX_LEN     64 // packets

//The maximum period of time that a routing protocol is 
//allowed to buffer a packet for.
#define AODV_RTQ_TIMEOUT     30	// seconds

typedef std::list<Packet*> PktList_t; // List of pending packets

class AODVQueue {
 public:
    AODVQueue();
    void  enque(Packet *p);

    // Returns a packet from the head of the queue.
    Packet* deque(void);
    // Returns a packet for destination "D".
    Packet* deque(IPAddr_t dst);
    // Finds whether a packet with destination dst 
	// exists in the queue
    char find(IPAddr_t dst);

 private:
    void purge(void);

    PktList_t  pkts; // List of packets in the queue
    int        nLimit; //the queque length limit
    Time_t     timeout;

};

#endif /* __aodv_queue_h__ */
