//*********************************************************************
//
// AUTHOR:    George Papageorgiou <gpapag@eng.umd.edu>
//
// FILE:      appAlert.h
//
// DATE:      October 27, 2003
//
// PURPOSE:   A new application that implements the alert protocol
//           for the dynamic firewall.
//
//*********************************************************************
/*
 * $Id: appAlert.h,v 1.1.1.1 2006/12/13 13:09:35 dnguyen Exp $
 */

// Converted to GTNetS by George F. Riley.
// Spring, 2004

#ifndef  _appAlert_h
#define  _appAlert_h

#include <cstdlib>
#include "timer.h"
#include "packet.h"
#include "application.h"

#include "cmnAlert.h"
#include "udpAlert.h"


#define   ALERT_OK            0
#define   ALERT_IPSPF         1
#define   ALERT_TCPFL         2
#define   ALERT_UDPFL         3

class DropTailFilter;

// Alert Application Class Definition

class AppAlert : public Application, public TimerHandler {
 public:
  typedef enum { AlertPort = 9998} AlertPort_t;

  AppAlert();
  AppAlert(const AppAlert&);
  void Receive(Packet*, L4Protocol*, Seq_t);       // (Sender/Receiver)
  void Timeout(TimerEvent*);
  void SendAlertPacket(IPAddr_t, PortId_t);
  void SendAlertPacket(int, IPAddr_t, IPAddr_t, PortId_t);
  void Connect(IPAddr_t, PortId_t);
  void SetServer(bool ss) { server = ss;  }
  void SetRegister(bool r){ reg = r; }
  IPAddr_t GetServerIP() { return serverIP; }

  // Methods from application base class
  void AttachNode(Node*); // Note which node attached to
  Application* Copy() const;// Make a copy of the application


protected:
  //int  command(int argc, const char*const* argv);
  void StartApp();          // Start sending data packets (Sender)
  void StopApp();           // Stop sending data packets (Sender)
public:
  void Init();
  void SetDTFQueue(DropTailFilter* dtf) { dtfQueue = dtf; }

private:
  void ServerRxAlert(AlertHeader*);
  void ClientRxAlert(AlertHeader*);

  UDP*      l4Proto;       // UDP protocol to send/receive pkts
  Node*     localNode;     // Attached noe
  int       running;       // If 1 application is running
  int       alertCode;     // the alert code
  int       attackerId;    // attacker's id
  bool      server;        // true if it is a server, false otherwise
  bool      reg;           // true if need to register with server
  IPAddr_t  serverIP;      // IP of attached server if not server
  DropTailFilter* dtfQueue;// Pointer to DTR queue,  if associated exists
  TargetVec_t targetList;    // list of registered clients
};


#endif

