/*

* GTNetS provides a portable C++ class library for network simulations
*
* Copyright (C) 2003 George F. Riley
*
* No guarantees or warranties or anything are provided or implied in any way
* whatsoever.  Use this program at your own risk.  Permission to use this
* program for any purpose is given, as long as the copyright is kept intact.
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

* Permission is hereby granted for any non-commercial use of this code

* $Id: application-cbr.h,v 1.1.1.1 2006/12/13 13:09:35 dnguyen Exp $

*/

// Georgia Tech Network Simulator - Constant Bit Rate Application class
// George F. Riley.  Georgia Tech, Spring 2002

// Define a CBR Application class

#ifndef __cbr_application_h__
#define __cbr_application_h__

#include "common-defs.h"
#include "application.h"
#include "notifier.h"
#include "udp.h"

#define DEFAULT_RATE 500000.0
#define DEFAULT_SIZE 512

// Define the start/stop events
class AppCBREvent : public Event {
public:
  typedef enum { SEND_PKT = 100 } AppCBREvent_t;
  AppCBREvent() { }
  AppCBREvent(Event_t ev) : Event(ev) { }
};

//Doc:ClassXRef
class CBRApplication : public Application, public NotifyHandler {
  //Doc:Class Define a application that generates constant bit rate data
  //Doc:Class between two nodes.
public:
  //Doc:Method
  CBRApplication(IPAddr_t, PortId_t, PortId_t = NO_PORT,
                 Rate_t = DEFAULT_RATE,
                 Size_t = DEFAULT_SIZE,
                 const L4Protocol& = UDP());
    //Doc:Desc Constructs a new CBR Application.
    //Doc:Arg1 \IPA\ of remote  endpoint.
    //Doc:Arg2 Port number for remote endpoint
    //Doc:Arg3 Port number for local endpoint
    //Doc:Arg4 Rate to generate data
    //Doc:Arg4 (specifying 0 indicates "as fast as possible)
    //Doc:Arg5 Size of individual packets.
    //Doc:Arg6 Layer 4 Protocol to use.

  // Copy constructor
  CBRApplication(const CBRApplication&);
  // Destructor
  ~CBRApplication();      
  void Handle(Event*, Time_t);
  void Notify(void*);         // Called if packet tx notification specified
  virtual void StartApp();    // Called at time specified by Start
  virtual void StopApp();     // Called at time specified by Stop
  virtual void AttachNode(Node*); // Note which node attached to
  virtual Application* Copy() const;// Make a copy of the application
  virtual L4Protocol*  GetL4() const { return l4Proto;}
public:
  IPAddr_t      remoteIP;     // IP Address of remote endpoint
  PortId_t      remotePort;   // Port number for remote endpoint
  PortId_t      localPort;    // Local port number
  Rate_t        cbrRate;      // Rate that data is generated
  Size_t        pktSize;      // Size of packets
  L4Protocol*   l4Proto;      // Points to the specified Layer 4 protocol
  bool          connected;    // True if already connected
  AppCBREvent*  pendingEvent; // Pending SendPkt event
};

#endif

