#ifndef __kneigh_application_h__
#define __kneigh_application_h__

#include "common-defs.h"
#include "application.h"
#include "interface-wireless.h"

#define USE_CISCO_AIRONET350

#define K_PORT             12345

typedef std::map<IPAddr_t, int> TxPowerDataMap_t;
typedef enum { K_HELP = 0x80, K_BEACON = 0x00 } KMsgType_t;

// Define the start/stop events
class AppKNeighLevEvent : public Event {
public:
  typedef enum { TIMEOUT = 100, START = 101 } AppKNeighLevEvent_t;
  AppKNeighLevEvent() { }
  AppKNeighLevEvent(Event_t ev) : Event(ev) { }
};

struct KNeighLevHdr
{
  KMsgType_t    type;         // message type (BEACON or HELP)
  int           txPowerLevel; // coded transmit power level of source
  IPAddr_t      ip;           // source ip

  inline int size()
  {
    return sizeof(KMsgType_t) + sizeof(int) + sizeof(IPAddr_t);
  }
};

//Doc:ClassXRef
class KNeighLevApplication : public Application {
  //Doc:Class Define an application that implements the k-NeighLev
  //Doc:Class topology control algorithm.

public:
  KNeighLevApplication(Node *n, Time_t start);
  // specify the node the algorithm will operate on

  KNeighLevApplication(const KNeighLevApplication &);

  void Handle(Event*, Time_t);
  virtual void StartApp();    // Called at time specified by Start
  virtual void StopApp();     // Called at time specified by Stop
  virtual Application* Copy() const;// Make a copy of the application
  virtual L4Protocol*  GetL4() const { return l4Proto;}
  virtual void Receive(Packet*,L4Protocol*, Seq_t = 0);   // Data received
  double GetTxPower() { return KNeighLevApplication::txPower[myTxPowerLevel]; }

public:
  Node*              node;          // keep a handle on the node
  InterfaceWireless* iw;            // keep a handle on the interface
  IPAddr_t           ip;            // keep a handle on the ip address
  L4Protocol*        l4Proto;       // Points to the specified Layer 4 protocol
  AppKNeighLevEvent* pendingEvent;  // Pending timeout event
  int                myTxPowerLevel;// Index into txPower array (0-127)
  TxPowerDataMap_t   Nsymmetric;    // symmetric neighbor list
  TxPowerDataMap_t   Nincoming;     // incoming neighbor list
  Time_t startTime;

  static double GetTxPowerFromRange(Meters_t range);

  static double txPower[];
  static double MAX_POWER;
  static Meters_t radioRange[];
  static int K_CONST;
  static double TL[];

private:
  void SendMessage(KMsgType_t type);
  void SetTxPowerLevel(int level);
  void StepWiseIncrease(int, int);

  bool changedPower;
};

#endif

