/*

* GTNetS provides a portable C++ class library for network simulations
*
* Copyright (C) 2003 George F. Riley
*
* No guarantees or warranties or anything are provided or implied in any way
* whatsoever.  Use this program at your own risk.  Permission to use this
* program for any purpose is given, as long as the copyright is kept intact.
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

* Permission is hereby granted for any non-commercial use of this code

* $Id: application-ping.h,v 1.1.1.1 2006/12/13 13:09:35 dnguyen Exp $

*/

// Georgia Tech Network Simulator - Ping Application class
// George F. Riley.  Georgia Tech, Summer 2004

// Define a Ping Application class

#ifndef __ping_application_h__
#define __ping_application_h__

#include "common-defs.h"
#include "application.h"
#include "icmp.h"

class IPV4Header;

// Define the start/stop events
class AppPingEvent : public Event {
public:
  typedef enum { SEND_PING = 100 } AppPingEvent_t;
  AppPingEvent() { }
  AppPingEvent(Event_t ev) : Event(ev) { }
};

//Doc:ClassXRef
class PingApplication : public Application {
  //Doc:Class Define a application that generates ICMP ECHO messages
public:
  //Doc:Method
  PingApplication(IPAddr_t);
    //Doc:Desc Constructs a new Ping Application.
    //Doc:Arg1 \IPA\ of Ping target

  // Copy constructor
  PingApplication(const PingApplication&);
  // Destructor
  ~PingApplication();      
  void Handle(Event*, Time_t);
  virtual void Receive(Packet*,L4Protocol*, Seq_t = 0);   // Data received
  virtual void StartApp();    // Called at time specified by Start
  virtual void StopApp();     // Called at time specified by Stop
  virtual void AttachNode(Node*); // Note which node attached to
  virtual Application* Copy() const;// Make a copy of the application
  virtual L4Protocol*  GetL4() const { return nil;}
  // Set defaults for all Ping apps
  static void DefaultPingCount(Count_t c) { defaultPingCount = c; }
  static void DefaultPingRate(Rate_t c) { defaultPingRate = c; }
  // Subclasses can override the below to take additional action on Reply.
  // NOTE.  Packet pointer will be owned by overridden subroutine, which
  // must delete it or pass it to another subroutine.
  virtual void PingReplyRx(Packet*, IPV4Header*, ICMPHeader*);
public:
  IPAddr_t      remoteIP;     // IP Address of Ping target
  Node*         localNode;    // Local node for applicatoin
  AppPingEvent* pendingEvent; // Pending SendPing event
  Count_t       pingCount;    // Number pings to send (0 = infinity)
  Rate_t        pingRate;     // Pings per second
  Count_t       pingSeq;      // Ping sequence number
private:
  // Default values for  parameters
  static Count_t defaultPingCount;
  static Rate_t  defaultPingRate;
};

#endif

