/*

* GTNetS provides a portable C++ class library for network simulations
*
* Copyright (C) 2003 George F. Riley
*
* No guarantees or warranties or anything are provided or implied in any way
* whatsoever.  Use this program at your own risk.  Permission to use this
* program for any purpose is given, as long as the copyright is kept intact.
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

* Permission is hereby granted for any non-commercial use of this code

*/

// Georgia Tech Network Simulator - TCP Server Application class
// George F. Riley.  Georgia Tech, Summer 2002

// Define a TCP base class for all appliations that use a single TCP L4Protocol
// Saves some code duplication the "application-tcp*.cc" classes

//#define DEBUG_MASK 0x01
#include "debug.h"
#include "application-tcp.h"
#include "l4protocol.h"
#include "tcp.h"
#include "simulator.h"

using namespace std;

// Constructor
TCPApplication::TCPApplication(const TCP& t)
  : l4Proto((TCP*)t.Copy()), closeOnEmpty(false)
{ 
  l4Proto->AttachApplication(this);
}

// Copy constructor
TCPApplication::TCPApplication(const TCPApplication& c)
  : Application(c), l4Proto((TCP*)(c.l4Proto->Copy())),
    closeOnEmpty(c.closeOnEmpty)
{
  l4Proto->AttachApplication(this);
}

// Destructor
TCPApplication::~TCPApplication()
{
  if (l4Proto)
    { // Don't delete if bound to a port.  Will self delete later
      // when timeout period is done
      if (l4Proto->localPort == NO_PORT)
        delete l4Proto;
    }
}

void TCPApplication::Bind(PortId_t p)
{
  GetL4()->Bind(p);
}

void TCPApplication::SetTrace(Trace::TraceStatus ts)
{ // Set the trace status of layer 4 protocol
  if (l4Proto) l4Proto->SetTrace(ts);
}

void TCPApplication::AttachNode(Node* n)
{ // Note attached node
  if (l4Proto) l4Proto->Attach(n);
}

// Private methods
void TCPApplication::Delete()
{ // Delete this application and remove connection to the TCP protocol object
  if (deleteOnComplete)
    {
      GetL4()->AttachApplication(nil);
      Simulator::instance->DeleteObject(this);
      DEBUG0((cout << "TCPApplication " << this
              << " deleting self" << endl));
    }
}
