/*

* GTNetS provides a portable C++ class library for network simulations
*
* Copyright (C) 2003 George F. Riley
*
* No guarantees or warranties or anything are provided or implied in any way
* whatsoever.  Use this program at your own risk.  Permission to use this
* program for any purpose is given, as long as the copyright is kept intact.
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

* Permission is hereby granted for any non-commercial use of this code

*/

// Georgia Tech Network Simulator - TCP Server Application class
// George F. Riley.  Georgia Tech, Summer 2002

// Define a TCP base class for all appliations that use a single TCP L4Protocol
// Saves some code duplication the "application-tcp*.cc" classes

#ifndef __application_tcp_h__
#define __application_tcp_h__

#include "common-defs.h"
#include "application.h"
#include "tcp.h"

class TCP;

//Doc:ClassXRef
class TCPApplication : public Application {
  //Doc:Class Class {\tt TCPApplication} defines a virtual base class that 
  //Doc:Class can be subclassed by any application that has a single {\tt TCP}
  //Doc:Class connection endpoint.
public:
  //Doc:Method
  TCPApplication(const TCP& t);
  //Doc:Class Constructor for {\tt TCPApplication}.
    //Doc:Arg1 Reference to any {\tt TCP} variation to use for the
    //Doc:Arg1 {\tt TCP} connection.

  //Doc:Method
  TCPApplication(const TCPApplication& c);
    //Doc:Desc Copy Constructor.
    //Doc:Arg1 {\tt TCPApplication} to copy.

  virtual ~TCPApplication();
  virtual Application* Copy() const = 0;  // Make a copy of this application

  //Doc:Method
  virtual void AttachNode(Node*); // Note which node attached to
    //Doc:Desc Specify which node to which this application is attached.
    //Doc:Arg1 Node pointer to attached node.

  //Doc:Method
  L4Protocol* GetL4() const { return l4Proto;}
    //Doc:Desc Get a pointer to the single {\tt TCP} object for this
    //Doc:Desc application.
    //Doc:Return Pointer to associated {\tt TCP} object.

  //Doc:Method
  void        Bind(PortId_t p);
    //Doc:Desc Bind the {\tt TCP} object to a specified port.
    //Doc:Arg1 Port number to bind.  Specify zero if an available port is to
    //Doc:Arg1 be chosen.

  //Doc:Method
  void        CloseOnEmpty(bool coe = true) { closeOnEmpty = coe;}
    //Doc:Desc Specify that the {\tt TCP} object should close the connection
    //Doc:Desc automatically when all pending data has been sent.
    //Doc:Arg1 True of close on empty desired.

  //Doc:Method
  void        SetTrace(Trace::TraceStatus);
    //Doc:Desc Set the trace status of the associated {\tt TCP} endpoint.
    //Doc:Arg1 Trace status desired.

protected:
  void        Delete();          // Delete this application
public:
  TCP*        l4Proto;
  bool        closeOnEmpty;
};

#endif
