/*

* GTNetS provides a portable C++ class library for network simulations
*
* Copyright (C) 2003 George F. Riley
*
* No guarantees or warranties or anything are provided or implied in any way
* whatsoever.  Use this program at your own risk.  Permission to use this
* program for any purpose is given, as long as the copyright is kept intact.
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

* Permission is hereby granted for any non-commercial use of this code

* $Id: application-traceroute.h,v 1.1.1.1 2006/12/13 13:09:35 dnguyen Exp $

*/

// Georgia Tech Network Simulator - Traceroute Application class
// George F. Riley.  Georgia Tech, Summer 2004

// Define a Traceroute Application class

#ifndef __traceroute_application_h__
#define __traceroute_application_h__

#include <vector>

#include "common-defs.h"
#include "application.h"
#include "icmp.h"

class IPV4Header;

// Define the start/stop events
class AppTracerouteEvent : public Event {
public:
  typedef enum { SEND_TRACE = 100 } AppTracerouteEvent_t;
  AppTracerouteEvent() { }
  AppTracerouteEvent(Event_t ev) : Event(ev) { }
};

typedef std::vector<Time_t> TimeVec_t; // For logging rtt to targets

//Doc:ClassXRef
class TracerouteApplication : public Application {
  //Doc:Class Define a application that generates ICMP ECHO messages
public:
  //Doc:Method
  TracerouteApplication(IPAddr_t);
    //Doc:Desc Constructs a new Traceroute Application.
    //Doc:Arg1 \IPA\ of Traceroute target

  // Copy constructor
  TracerouteApplication(const TracerouteApplication&);
  // Destructor
  ~TracerouteApplication();      
  void Handle(Event*, Time_t);
  virtual void Receive(Packet*,L4Protocol*, Seq_t = 0);   // Data received
  virtual void StartApp();    // Called at time specified by Start
  virtual void StopApp();     // Called at time specified by Stop
  virtual void AttachNode(Node*); // Note which node attached to
  virtual Application* Copy() const;// Make a copy of the application
  virtual L4Protocol*  GetL4() const { return nil;}
  // Subclasses can override the below to take additional action on Reply.
  // NOTE.  Packet pointer will be owned by overridden subroutine, which
  // must delete it or pass it to another subroutine.
  virtual void TracerouteReplyRx(Packet*, IPV4Header*, ICMPHeader*);
private:
  void SendProbePacket();
protected:
  virtual void PrintResultsLine();
public:
  IPAddr_t      remoteIP;     // IP Address of Traceroute target
  Node*         localNode;    // Local node for applicatoin
  AppTracerouteEvent* pendingEvent; // Pending SendTraceroute event
  Count_t       hopCount;     // Current hop count
  Count_t       retryCount;   // Current retry count
  Count_t       probeCount;   // Number of probes per target
  Count_t       maxTTL;       // Maximum ttl to use
  Time_t        timeSent;     // Time the probe was sent
  IPAddr_t      respondIP;    // Responder's IP addres
  bool          done;         // True if got UNREACHABLE (from target)
  Time_t        timeout;      // Timeout value (sec) for waiting replies
  TimeVec_t     rttHist;      // Measured RTT for each of the probes
};

#endif

