/*

* GTNetS provides a portable C++ class library for network simulations
*
* Copyright (C) 2003 George F. Riley
*
* No guarantees or warranties or anything are provided or implied in any way
* whatsoever.  Use this program at your own risk.  Permission to use this
* program for any purpose is given, as long as the copyright is kept intact.
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
A
* Permission is hereby granted for any non-commercial use of this code

*/

// Georgia Tech Network Simulator - Worm Application

// Monirul I Sharif,  Georgia Tech, Fall 2003

// Define a Worm Application class

// Uncomment below to enable debug level 0

//#define DEBUG_MASK 0x1

#include "debug.h"
#include "application-worm.h"
#include "udp.h"
#include "node.h"
#include "simulator.h"

#ifdef HAVE_QT
#include <qnamespace.h>
#endif

using namespace std;


Count_t  WormApplication::defaultscanspersecond = 10;
Size_t   WormApplication::defaultinfectionlength = 376;
PortId_t WormApplication::defaultinfectionport = 1434;
//100% machines are vulnerable
double   WormApplication::defaultvulnerability = 1.0;
IPAddr_t WormApplication::defaultbaseIP = IPAddr("192.168.0.0");
Count_t  WormApplication::defaultIPscanrange = 65535;

// In this version of the worm application I am using a constant
//    packets per second method to sent UDP packets.
// I will later change this into a more robust method.

// Constructors


WormApplication::WormApplication(Node* n)
    : infected(false), vulnerable(true), udp(nil), node(nil),
    rngT(nil), timeoutevent(nil),
    infectionlength(defaultinfectionlength),
    scanspersecond(defaultscanspersecond),
    IPscanrange(defaultIPscanrange)
{
  DEBUG0((cout << "Attaching application " << this << endl));
}


// Application Methods
void WormApplication::StartApp()    // Called at time specified by Start
{
  if (infected)
	StartInfection();
}

void WormApplication::StopApp()     // Called at time specified by Stop
{
  if (timeoutevent) timer.Cancel(timeoutevent);
}

void WormApplication::AttachNode(Node* n) 
{
  udp = new UDP(n);
  
  udp->Attach(node);
  udp->Bind(defaultinfectionport);
  udp->AttachApplication(this);
  DEBUG0((cout << "udp " << udp << " , attach App " << this << endl));
  node = n;
  if(udp) { 
    udp->Attach(node);
    udp->Bind(defaultinfectionport);
  }
}

Application* WormApplication::Copy() const
{
  return new WormApplication(*this);
}

void WormApplication::Infect()
{
  infected = true;
}

void WormApplication::StartInfection()
{
  DEBUG0(( cout<<"Infected machine:"<< IPAddr::ToDotted(node->GetIPAddr())<<endl ));
  infected = true;
  DEBUG0(( cout<<" Now starting timer event"<<endl));
#ifdef HAVE_QT
  // Set the node to red on animation
  node->Color(Qt::red);
#endif
  ScheduleNextPacket();
}


bool WormApplication::IsInfected()
{
  return infected;
}

void WormApplication::SendWormPacket()
{
  //  char buffer[infectionlength+1];   // later on I'll add the real contents here
  
  IPAddr_t target = GenerateNextIPAddress();  
  DEBUG0((cout<<"sending worm packet to "<<IPAddr::ToDotted(target)<<endl));
  udp->SendTo(infectionlength, target, defaultinfectionport);
  //  DEBUG0((cout<<"sent worm packet"<<endl));  
}

IPAddr_t WormApplication::GenerateNextIPAddress()
{

  //   randomx = randomx * 214013 + 0xffd9613c;
  // I am not following the random number generator of the Slammer worm
  IPAddr_t IP;

  if (!rngT) rngT = new Uniform(0, IPscanrange);
  IP = defaultbaseIP + rngT->IntValue();

  return IP;
}

void WormApplication::SetDefaultScanRate(Count_t rate)
{
  defaultscanspersecond = rate;
}

void  WormApplication::SetDefaultBaseIP(IPAddr_t base)
{
  defaultbaseIP = base;
}

void WormApplication::SetDefaultScanRange(Count_t range)
{
  defaultIPscanrange = range;
}

void WormApplication::Receive(Packet *p, L4Protocol *proto, Seq_t)
{
   DEBUG0((cout<<"Received UDP packet"<<endl));

  if (PacketIsWorm(p)) {
    if (vulnerable && !infected) {
      StartInfection();
    }
  }
  delete p;
}


// Right now every packet received is a worm
bool WormApplication::PacketIsWorm(Packet *p)
{
  return true;
}

void WormApplication::ScheduleNextPacket()
{
  if (!timeoutevent) timeoutevent = new TimerEvent();
  timer.Schedule(timeoutevent, 1.0 / (double)scanspersecond, this);
}

void WormApplication::Timeout(TimerEvent *ev)
{
  if (!node)
    {
      cout << "WormApplication::Timeout with no attached node "<<endl;
      return;
    }

  DEBUG0((cout<< "WormApplicationWorm::Timeout "<<Simulator::Now()<<endl));

  // Sending the worm packet
  SendWormPacket();
  ScheduleNextPacket();
}
