/*

* GTNetS provides a portable C++ class library for network simulations
*
* Copyright (C) 2003 George F. Riley
*
* No guarantees or warranties or anything are provided or implied in any way
* whatsoever.  Use this program at your own risk.  Permission to use this
* program for any purpose is given, as long as the copyright is kept intact.
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

* Permission is hereby granted for any non-commercial use of this code

*/

// Define a bitmap object
// George F. Riley, Georgia Tech, Winter 2000

// Creates a bitmap object.  The 'entries' can be more than one bit,
// but default to one bit.  Bits per entry can't be more than 32.
// Entries DO NOT span words, for ease of coding

#ifndef __BITMAP_H__
#define __BITMAP_H__

#include <iostream>

#include "common-defs.h"

const unsigned  int BITS_ULONG = 32;

//Doc:ClassXRef
class BitMap {
  public :
    BitMap      ( );
    BitMap      ( Count_t Size, Count_t BitsPerEntry = 1);
    ~BitMap     ();
    void    Set  ( Count_t Which, Count_t Value = 1);
    void    Clear( Count_t Which);
    Count_t Get  ( Count_t Which);  // Return the specified entry
    size_t  Size ( void );         // Return storage used
    void    Log  ( std::ostream& os);   // Log to ostream
    void    DBPrint();
    static Count_t FindBPE( Count_t );
    static size_t  EstimateSize( Count_t Size, Count_t BitsPerEntry);
  private :
    Count_t*  GetWordAddress(Count_t Which); // Get pointer to the word needed
    Count_t   GetBitMask(Count_t Which);     // Get a bit mask for needed bits
    short     GetShiftCount( Count_t Which); // Get a shift count for position
    void      Validate(Count_t Which);       // Validate which not too large
    Count_t   size; // how many entries
    Count_t   BPE;  // Bits per entry
    Count_t   words;// Number of words needed for bitmap
    short     EPW;  // Entries per word
    Bitmap_t* pM;   // Pointer to the actual map (or the data if < 32 bits)
};

#endif
