//*********************************************************************
//
// AUTHOR:    George Papageorgiou <gpapag@eng.umd.edu>
//
// FILE:      cmnAlert.h
//
// DATE:      October 27, 2003
//
// PURPOSE:   Contains common definitions for the application and the
//           agent.
//
//*********************************************************************

// Converted to GTNetS by George Riley, Georgia Tech
// Spring 2004
#ifndef _cmnAlert_h
#define _cmnAlert_h

#include <vector>

#include "pdu.h"
class TargetAddrPort
{
public:
  TargetAddrPort(IPAddr_t i, PortId_t p) : ip(i), port(p) {}
  IPAddr_t ip;
  PortId_t port;
};


typedef std::vector<TargetAddrPort> TargetVec_t;

// Define the interface between the agent and the application
// Converted to GTNetS PDU
class AlertHeader : public Data
{
public:
  AlertHeader()
      : srcIP(0), srcPort(0), attackerId(0),
        alertCode(0), nbytes(0), time(Simulator::Now()) {}
  AlertHeader(const AlertHeader& c)
      : srcIP(c.srcIP), srcPort(c.srcPort), attackerId(c.attackerId),
      alertCode(c.alertCode), nbytes(c.nbytes), time(c.time) {}
  Size_t Size() const { return nbytes; }
  PDU*   Copy() { return new AlertHeader(*this); }
  Layer_t Layer() { return 5; } // Application layer
  Proto_t Proto() { return 1001; } // Made up code for Alert protocol
  // Serialization not implemented yet
public:
  int  srcIP;              // source ip
  int  srcPort;            // source port
  int  attackerId;         // source ip of the attacker
  int  alertCode;          // alert code
  int  nbytes;             // bytes for packet
  double time;             // current time
};



#endif


