/*

* GTNetS provides a portable C++ class library for network simulations
*
* Copyright (C) 2003 George F. Riley
*
* No guarantees or warranties or anything are provided or implied in any way
* whatsoever.  Use this program at your own risk.  Permission to use this
* program for any purpose is given, as long as the copyright is kept intact.
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

* Permission is hereby granted for any non-commercial use of this code

*/

/* 
 *
 * dfs.c -- simple dfs program and routine that uses dfs to check connectivity
 *
 * Uses the Stanford GraphBase tools.
 */

#include <stdio.h>
#ifndef FBSD
#include <alloca.h>
#endif
#include <sys/types.h>
#include "gb_graph.h"
#include "math.h"
#include "geog.h"

#define STACKMAX	0x600

#define NBBY 8

/* check connectivity of graph g                        */
/* uses depth-first search.                             */
isconnected(Graph *G)
{
u_char *vis;
Vertex *vp;
int i,nbytes;

    nbytes = (G->n/NBBY)+ (G->n%NBBY?1:0);

    if (nbytes < STACKMAX) {	/* for small amounts we use stack frame */
        vis = (u_char *) alloca(nbytes);
    } else {
        vis = (u_char *) malloc(nbytes);
    }

    for (i=0; i<nbytes; i++) vis[i]=0;

    i =  (dfs(G,0,vis)==G->n);

    if (nbytes >= STACKMAX) free(vis);

    return i;

}

#define bitset(v,i)     (v[(i)/NBBY]&(1<<((i)%NBBY)))
#define setbit(v,i)     v[(i)/NBBY] |= (1<<((i)%NBBY))


/* trivial depth-first search. */
/* Returns number of newly-visited nodes in subtree rooted at node n. */
int
dfs(Graph *G,int n,u_char *vis)
{
int nvis, i, nextn;
Vertex *vp;
Arc *ap;


    setbit(vis,n);       /* mark this node as visited */
    vp = G->vertices + n;
    nvis = 1;           /* nvis == number of nodes visited */
    for (ap=vp->arcs; ap; ap=ap->next) {
         nextn = ap->tip - G->vertices;
        if (!bitset(vis,nextn)) {  /* not seen yet */
            nvis += dfs(G,nextn,vis);
        }
    }
    return nvis;
} /* dfs */
