/*

* GTNetS provides a portable C++ class library for network simulations
*
* Copyright (C) 2003 George F. Riley
*
* No guarantees or warranties or anything are provided or implied in any way
* whatsoever.  Use this program at your own risk.  Permission to use this
* program for any purpose is given, as long as the copyright is kept intact.
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

* Permission is hereby granted for any non-commercial use of this code

*/

// Georgia Tech Network Simulator - DropTail Queue class with spoof filtering
// George F. Riley.  Georgia Tech, Spring 2004

#ifndef __droptailfilter_h__
#define __droptailfilter_h__

#include <vector>
#include <map>

#include "droptail.h"
#include "udp.h"
#include "appAlert.h"
#include "timer.h"
#include "node.h"

#ifdef HAVE_QT
#include <qcolor.h>
#endif

// From UMd Code
#define BOUNDC          4    // max number of links per router
#define BOUNDR          30   // Number of refresh rates for computing historic 
		             // estimate...  BOUNDR<LOADSIZE
#define LOADSIZE        100
#define FALSEALARMDELAY 20
// End UMd

// Added by GFR to support UMd Code
typedef std::vector<int>        LoadVec_t;
typedef std::map<IPAddr_t, int> ThroughputMap_t;

class DropTailFilter : public DropTail, public TimerHandler {
public:
  DropTailFilter();
  DropTailFilter(Count_t); // Constructor with size
  DropTailFilter(const DropTailFilter&); // Copy constructor
  virtual ~DropTailFilter();
  Queue*  Copy() const;       // Make a copy
  bool Enque(Packet*); // Return true if enqued, false if dropped
  bool CheckSpoofedSource(Packet*); // Check for spoofed source address
  void SetRateAlgorithm(bool);

  // From TimerHandler
  void Timeout(TimerEvent*);

  // Specific to DropTailFilter
  void Connect(IPAddr_t, PortId_t); // Connection to server
  void ValidIP(IPAddr_t, Mask_t);   // Specify valid source IP's
private:
  void ConstructorHelper();
  void RateAlgorithm(Packet* p);
  void DropPacket(Packet* p, const char*);
  double ComputeStatistic();
public:
  UDP*      l4Proto;    // UDP proto to use to contact server
  IPAddr_t  validIP;    // Valid IP's allowed on link (for spoof detection)
  Mask_t    validMask;  // Mask for above
  IPAddr_t  filterIP;   // IPAddress of mis-behaving host to always drop
  AppAlert* appAlert;   // Alert application for notifying others of storms
  bool      needAlert;  // True if need an alert on next spoofed pkt
  bool      rateAlgorithm; // true if need to run the rx-rate algorithm
  Timer     timer;      // Timer to re-enable alerts
  QColor    normalColor;// Original color of node for restoration

  // Stuff for attack detection from UMd
  // Added by gpapag May 6, 2003 for multiple link detection
  int   fromNode_;            // the node that sends to the link
  int   toNode_;              // the node that receives from the link
  int   color_;               // the color of the queue
  // Added by gpapag for IP spoofing
  int  IPSpoofingChecked;
  // Added by gpapag for UDP flooding
  IPAddr_t UDPfloodingIP;
  bool     UDPchecked;
  //int  posG;
  //jjs DDOS-----------------
  bool DDOS_on;
  int  LL;
  LoadVec_t Load;
  double LastUpDate;
  int  Rate[100];
  int  L_index; 
  //ALVARO DDOS----------------------------------------------
  double y[BOUNDR][BOUNDC];	// where r is the bound for r
  int gamma;	                // which link are we computing
  double meany[BOUNDC];	        // mean of y. 10 is a bound on the 
  // number of outgoing links
  double covy[BOUNDC];	      //uncorrelated covariance matrix->vector
  int r;                //length of time window to compute the statistic
  int c;	                        // number of links at router
  double historicy[BOUNDR][BOUNDC];
  double nonparamstat;
  double YellowThres;
  double RedThres;
  
  //ALVARO DDOS----------------------------------------------
  // jjs Engress Filtering-----------------------------------
  
  ThroughputMap_t ThroughputPerSource; // rate of each source after alarm
  IPAddr_t MaxRateSourceIpAddr;       // block traffic from the source ip addr
  int TotalArriveAfterAlarm;     // no. of packet arrive > some fix 
                                 // integer (ex. 2000) -->begin to block 
                                 // after alarm
  bool UnderAttack;
  // jjs  Engress Filtering----------------------------------
  // Added from Jia-Shiang's code for tcp-flooding
#ifdef CHECK_SYN_FLOOD
  // Not presently used
  int       Firewall_;
  int       Monitor_Attacker[1000];
  SYNTimer *Monitor_Timer[1000];
#endif
  // The tcl scripts that run this code in ns2 define several global variables
public:
  static int Size;
  static int Degree;
  static int Subnets;
  static int SizeOfSubnets;
};

#endif
  
