/*

* GTNetS provides a portable C++ class library for network simulations
*
* Copyright (C) 2003 George F. Riley
*
* No guarantees or warranties or anything are provided or implied in any way
* whatsoever.  Use this program at your own risk.  Permission to use this
* program for any purpose is given, as long as the copyright is kept intact.
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

* Permission is hereby granted for any non-commercial use of this code

*/

// DropTail Queue class with IPA derivatives calculation
// This version calculates derivatives when the sources retransmit
// lost packets.
// George F. Riley, Georgia Tech, Spring 2004

#ifndef __droptail_ipa_retx_h__
#define __droptail_ipa_retx_h__

#include "droptail.h"
#include "handler.h"
#include "event.h"

class ReTxEvent : public Event 
{
public:
  ReTxEvent() : weight(0), p(0) {}
public:
  Count_t weight; // Derivative weight
  Packet* p;      // Packet to re-transmit
};

//Doc:ClassXRef
class DropTailIPAReTx : public DropTail, public Handler {
public :
  DropTailIPAReTx();
  DropTailIPAReTx(Count_t l);              // Specify limit
  DropTailIPAReTx(const DropTailIPAReTx&); // Copy constructor
  ~DropTailIPAReTx();                      // Destructor
  // Handler functions
  virtual void Handle(Event*, double);     // Event handler
  // DropTail functions
  bool Enque(Packet*);               // Return true if enqued, false if dropped
  Packet*  Deque();                  // Return next element (NULL if none)
  Queue*   Copy() const;             // Make a copy
  // New functions for IPA queues
  virtual  NCount_t DL1dB1();        // Get dl1db1 derivative
  virtual  Time_t   DQ1dB1();        // Get dq1db1 derivative
  Count_t           Losses();        // Total losses
  Count_t           TotalEnque() { return totEnq; }
  
private:
  bool    busyPeriod; 
  bool    lossyBusyPeriod;
  bool    full;              // True if queue is full
  Count_t totalLosses;       // Total packet losses
  Count_t c1;                // C1 counter
  Count_t c2;                // C2 counter
  Count_t dldq;              // Derivative of loss wrt queue  limit
  Count_t retxWeight;        // Weight needed for re-tx event
  bool    retx;              // True if next Enque is a retransmit
  Count_t startWeight;       // Weight of packet starting LBP
public:
  Count_t debugCount;        // debug
public:
  static  Time_t retxTime;   // Retransmit delay
};

#endif

