/*

* GTNetS provides a portable C++ class library for network simulations
*
* Copyright (C) 2003 George F. Riley
*
* No guarantees or warranties or anything are provided or implied in any way
* whatsoever.  Use this program at your own risk.  Permission to use this
* program for any purpose is given, as long as the copyright is kept intact.
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

* Permission is hereby granted for any non-commercial use of this code

*/

// Georgia Tech Network Simulator - Event class
// George F. Riley.  Georgia Tech, Spring 2002

#ifndef __event_h__
#define __event_h__

#include "debug.h"
#include "common-defs.h"
#include "globalstats.h"
#include "memreuse.h"

class Handler;

#define NO_EVENT 0xffffffff
//Doc:ClassXRef
class Event : public ReuseBase {
public:
  Event() 
    : event(NO_EVENT), uid(0), time(0), handler(0)
    {Stats::eventsAllocated++;}
  Event(Event_t ev) 
    : event(ev), uid(0), time(0), handler(0)
    {Stats::eventsAllocated++;}
  Event(Event_t ev, Handler* h) 
    : event(ev), uid(0), time(0), handler(h)
    {Stats::eventsAllocated++;}
  virtual ~Event() { Stats::eventsDeleted++; };
  inline Time_t         Time() const { return time;}
  inline void           Time(Time_t t){  time = t; }
public:
  Event_t        event;
  SimulatorUid_t uid;
private:
  Time_t         time;
public:
  Handler*       handler;
};
#endif
