/*

* GTNetS provides a portable C++ class library for network simulations
*
* Copyright (C) 2003 George F. Riley
*
* No guarantees or warranties or anything are provided or implied in any way
* whatsoever.  Use this program at your own risk.  Permission to use this
* program for any purpose is given, as long as the copyright is kept intact.
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

* Permission is hereby granted for any non-commercial use of this code

*/

// Georgia Tech Network Simulator - Empirical Distributions for HTTP models
// George F. Riley.  Georgia Tech, Spring 2002

// Defines random variables for HTTP Web Browsing models.
// Based on study by Bruce Mah, InfoComm'97
// See Bruce's paper for details on each of these distributions

#ifndef __http_distributions_h__
#define __http_distributions_h__

#include "rng.h"

//Doc:ClassXRef
class HttpPrimaryRequest : public IntEmpirical {
public:
  HttpPrimaryRequest(Seed_t s = RANDOM_SEED);
};

//Doc:ClassXRef
class HttpSecondaryRequest : public IntEmpirical {
public:
  HttpSecondaryRequest(Seed_t s = RANDOM_SEED);
};

//Doc:ClassXRef
class HttpPrimaryReply : public IntEmpirical {
public:
  HttpPrimaryReply(Seed_t s = RANDOM_SEED);
};

//Doc:ClassXRef
class HttpSecondaryReply : public IntEmpirical {
public:
  HttpSecondaryReply(Seed_t s = RANDOM_SEED);
};

//Doc:ClassXRef
class HttpFilesPerPage : public IntEmpirical {
public:
  HttpFilesPerPage(Seed_t s = RANDOM_SEED);
};

//Doc:ClassXRef
class HttpConsecutivePages : public IntEmpirical {
public:
  HttpConsecutivePages(Seed_t s = RANDOM_SEED);
};

//Doc:ClassXRef
class HttpThinkTime : public Empirical {
public:
  HttpThinkTime(Seed_t s = RANDOM_SEED);
  void Bound(Random_t); // Specify an upper bound on the returned value
  Random_t Value();     // Override value to enforce bound
private:
  Random_t bound;
};

#endif



