/*

* GTNetS provides a portable C++ class library for network simulations
*
* Copyright (C) 2003 George F. Riley
*
* No guarantees or warranties or anything are provided or implied in any way
* whatsoever.  Use this program at your own risk.  Permission to use this
* program for any purpose is given, as long as the copyright is kept intact.
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

* Permission is hereby granted for any non-commercial use of this code

*/

// Georgia Tech Network Simulator - Ghost Interface class
// George F. Riley.  Georgia Tech, Spring 2002

// Define class to model "Ghost" network link interfaces
// Ghosts are used in distributed simulatioonsn to maintain
// connectivity information for nodes assigned to other
// processes.

#ifndef __interface_ghost_h__
#define __interface_ghost_h__

#include "common-defs.h"
#include "interface.h"

class L2Proto;
class Node;
class Queue;
class LinkEvent;

//Doc:ClassXRef
class InterfaceGhost : public Interface
{
public:
  InterfaceGhost(const L2Proto& l2 = L2Proto802_3(),
		 IPAddr_t i   = IPADDR_NONE,
		 Mask_t   m   = MASK_ALL,
		 MACAddr  mac = MACAddr::NONE,
		 bool bootstrap = false);
  virtual ~InterfaceGhost();
  bool Send(Packet*, IPAddr_t, int);         // Enque and transmit
  bool Send(Packet*, const MACAddr&, int);   // Enque and transmit
  bool Send(Packet*, Word_t);                // Enque and transmit
  virtual void Handle(Event*, Time_t);
  virtual void Notify(void*);
  Count_t      PeerCount() const;            // Number of peers
  IPAddr       PeerIP(int npeer) const;      // Get peer IP addr if known
  IPAddr       NodePeerIP(Node*) const;      // Get peer IP addr if known
  bool         NodeIsPeer(Node*) const;      // Find out if node is a peer
  Count_t      NodePeerIndex(Node*) const;   // Get peer index for node
  Link*        GetLink() const;              // Return the link pointer
  void         SetLink(Link*);               // Set a new link pointer
  Queue*       GetQueue() const;             // Return the queue pointer
  void         SetQueue(const Queue& q);     // Set a new queue object
  bool         QueueDetailed() const;        // True if queue is detailed
  L2Proto*     GetL2Proto() const;           // Layer 2 proto object
  void         SetL2Proto(L2Proto* l2);      // Set a new l2 protocol object
  void         AddNotify(NotifyHandler*,void*); // Add a notify client
  void         CancelNotify(NotifyHandler*); // Cancel pending notif.
  MACAddr      IPToMac(IPAddr_t) const;      // Return mac addr for ip
  Count_t      NeighborCount(Node*) const;   // Number of rtg neighbors
  void         Neighbors(NodeWeightVec_t&, bool forceAll = false);
  IPAddr_t     DefaultPeer();                // Get default peer from link
  bool         IsLocalIP(IPAddr_t) const;    // True if IP is on this i/f
  Mult_t       TrafficIntensity() const;     // TotBytesSent/LineSpeed
  void         AddRemoteIP(IPAddr_t, Mask_t, Count_t = 0);// Add to rmt IP list
  void         HandleLLCSNAP(Packet *p);     // Process LLCSnap header
  bool         IsWireless() const { return false;} // True if wireless interface
  bool         IsEthernet() const {return false;}  // True if ethernet interface
  bool         IsReal() const { return false;}     // True if not Ghost
  void         SetWormContainment(WormContainment*) {}
  WormContainment* GetWormContainment() { return nil;}

 private:
  Link*       pLink;         // Associated link
public:
  static Count_t count;    // debug
};

#endif
