/*

* GTNetS provides a portable C++ class library for network simulations
*
* Copyright (C) 2003 George F. Riley
*
* No guarantees or warranties or anything are provided or implied in any way
* whatsoever.  Use this program at your own risk.  Permission to use this
* program for any purpose is given, as long as the copyright is kept intact.
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

* Permission is hereby granted for any non-commercial use of this code

*/

// Georgia Tech Network Simulator - IPAddress Class
// George F. Riley.  Georgia Tech, Spring 2002

#include <iostream>
#include <vector>
#include <stdio.h>

#include "ipaddr.h"
#include "stringtok.h"

using namespace std;

static char dotted[16];    // Common buffer for dotted notation conversion

// Constructors

IPAddr::IPAddr(string& s) : ip(ToIP(s))
{
}

IPAddr::IPAddr(const char* c) : ip(ToIP(string(c)))
{
}

IPAddr::operator string() const
{
  return string(ToDotted(ip));
}

// Static methodds
IPAddr_t IPAddr::ToIP(const string s)
{
  unsigned long r = 0;
  vector<string> t;
  stringtok(t, s, ".");
  if (t.size() != 4)
    {
      cout << "Error in IP Dotted.  Expected a.b.c.d Found " << s << endl;
      return 0;
    }
  for (vector<string>::size_type i = 0; i < t.size(); ++i)
    { // Process each subfield
      int v = atol(t[i].c_str());
      r <<= 8;
      r += v;
    }
  return r;
}

char* IPAddr::ToDotted(IPAddr_t ip)
{
  sprintf(dotted, "%ld.%ld.%ld.%ld",
          (ip >> 24) & 0xff,
          (ip >> 16) & 0xff,
          (ip >> 8)  & 0xff,
          ip & 0xff);
  return dotted;
}
