/*

* GTNetS provides a portable C++ class library for network simulations
*
* Copyright (C) 2003 George F. Riley
*
* No guarantees or warranties or anything are provided or implied in any way
* whatsoever.  Use this program at your own risk.  Permission to use this
* program for any purpose is given, as long as the copyright is kept intact.
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

* Permission is hereby granted for any non-commercial use of this code

*/

// Simplified 802.11-ish layer (Boeing)

#ifndef __l2proto_simple_h
#define __l2proto_simple_h
#include <sys/types.h>

#include "math.h"
#include "l2proto.h"
#include "tfstream.h"
#include "macaddr.h"
#include "timer.h"
#include "rng.h"
#include "interface-wireless.h"
#include "l2proto802.11.h"

using namespace std;

//Doc:ClassXRef
class L2ProtoSimple : public L2Proto, public TimerHandler, public Handler {
  //Doc:Class The {\tt L2ProtoSimple } class derives from the {\tt L2Proto}
  //Doc:Class class and defines the behavior of the 802.11 MAC protocol

public:
  L2ProtoSimple(); // Constructor
  virtual ~L2ProtoSimple();
  virtual void BuildDataPDU(MACAddr, MACAddr, Packet*);
  virtual void DataRequest(Packet*);
  virtual void DataIndication(Packet*);    // from lower layer

  // These functions just need empty definitions, to compile
  virtual bool Busy() {return false;}
  L2Proto* Copy() const;
  bool IsWireless() const { return true;}
  virtual void Handle(Event* ev, Time_t t) {}
  virtual void Bootstrap() {}
  void Timeout(TimerEvent*) {}
  void Notify(void*) {}
  
  Rate_t          dataRate;
  Word_t          seqNo;  // data frame sequence number
  
private:
  long   usec(double t) { return long(floor((t *= 1e6)+0.5)); }
  double txtime(long a, double b) { return a*8/b; }

};

//Doc:ClassXRef
class L2HeaderSimple : public PDU  // base class for all 802.11 frames
{
public:
  //Doc:Method
  L2HeaderSimple () {}

  virtual Size_t Size() const = 0;
  //  PDU* Copy() {return new L2HeaderSimple(*this);}
#ifdef ETRACE
  virtual Proto_t Proto() { return 11;} // For 802.11        
  virtual Layer_t Layer() { return 2;}         // Indicate layer 2 pdu
#endif

};
#endif
