/*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
                                                                                
* Permission is hereby granted for any non-commercial use of this code
*/
                                                                                
// Glue code between ospf6d and gtnets
// Copyright 2004, The Boeing Company

// Functions that are ported specially for simulation

#ifndef __sim_h__
#define __sim_h__

#include <sys/time.h>
#include "common-defs.h"
#include "../interface.h"
#include "../routing.h" // for RoutingEntry
#include "thread.h"

#define OSPF_ALLSPFROUTERS              0xe0000005      /* 224.0.0.5 */
#define OSPF_ALLDROUTERS                0xe0000006      /* 224.0.0.6 */
#define ETRACE_MAXSTR  3000

struct ospf6_msg {
	OSPF6DInstance *the_instance;
	int sockId;
	struct msghdr *msgHdr;
	int flags;
	struct thread *thd;
};

int gettimeofday_sim(struct timeval *tv, struct timezone *tz);
void localtime_sim(struct tm *);
#ifdef OSPF6_JITTER
int sendmsg_sim(int s, const struct msghdr *msg, int flags, int msec_jitter);
#else
int sendmsg_sim(int s, const struct msghdr *msg, int flags); 
#endif //JITTER
int sendmsg_wrapper(struct thread *);

void ipv6_to_ipv4(struct in6_addr *ipv6, IPAddr_t *ipv4);
void ipv4_to_ipv6(IPAddr_t *ipv4, struct in6_addr *ipv6);
void if_config_sim (struct interface *ifp);
void connected_init(struct interface *ifp, Interface *i);
int setsockopt_sim(int s, int  level,  int  optname,  const  void  *optval,
                   int optlen);
int setsockopt_ipv6_pktinfo_sim(int sock, int val);
int rand_sim(void);
void LookupRoute_ospf6sim(IPAddr_t src, IPAddr_t dst,RoutingEntry* r );
//RoutingEntry LookupRoute_ospf6sim(IPAddr_t src, IPAddr_t dst);
void ospf6_route_table_show_sim (struct ospf6_route_table *table);
void ospf6_show_neighbors_sim();
char *if_indextoname_sim (unsigned int ifindex, char *name);

void sim_add_timeout_event(Time_t, struct thread *);
void sim_cancel_event(void *sim_ev);

void TraceEvent_sim(int level, const char *, ...);
bool collect_stats_sim(struct ospf6 *o);

#endif /* __sim_h__ */
