/*

* GTNetS provides a portable C++ class library for network simulations
*
* Copyright (C) 2003 George F. Riley
*
* No guarantees or warranties or anything are provided or implied in any way
* whatsoever.  Use this program at your own risk.  Permission to use this
* program for any purpose is given, as long as the copyright is kept intact.
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

* Permission is hereby granted for any non-commercial use of this code

*/

// Georgia Tech Network Simulator - Ghost Link class
// George F. Riley.  Georgia Tech, Spring 2002

#ifndef __link_ghost_h__
#define __link_ghost_h__

#include "link.h"
#include "interface.h"
#include "l2proto802.3.h"

//Doc:ClassXRef
class LinkGhost : public Link {
  //Doc:Class Class {\tt LinkGhost} defines a {\em Ghost} link
  //Doc:Class derived from the {\tt Link} object, which is used
  //Doc:Class for distributed simulations.  Ghost links are assigned to
  //Doc:Class ghost  interfaces, and simply maintain connectivity
  //Doc:Class information.
public:
  
  LinkGhost();
  LinkGhost(const LinkGhost&);
  virtual ~LinkGhost();
  Link*      Copy() const ;     // Make a copy of this link

  // Managing the link
  void       SetPeer(Interface* i) { pPeer = i;}
  Interface* GetPeer(Packet*) { return pPeer;}
  Interface* GetPeer(Count_t) { return pPeer;}
  
  MACAddr    IPToMac(IPAddr_t);
  bool       Transmit(Packet*, Interface*, Node*)
    { return false;}// Transmit a packet
  bool       Transmit(Packet*, Interface*, Node*, Rate_t)
    { return false;}
  bool       TransmitWithDelay(Packet*, Time_t) { return false;}
  void       Handle(Event*, Time_t)  {};
  void       Bandwidth(Rate_t b) { }
  Rate_t     Bandwidth() const   { return 0.0;}
  void       Delay(Time_t d)     { }
  Time_t     Delay() const       { return 0.0;}
  bool       Busy() const        { return false;}
  void       BitErrorRate(Mult_t b) { }
  Mult_t     BitErrorRate() { return 0.0;}
  void       ResetUtilization(){ };
  Mult_t     Utilization() { return 0.0;}
  void       AddNotify(NotifyHandler*,void*){ }// Add a notify client
  Weight_t   Weight() { return 0.0;}
  void       Weight(Weight_t) { }
  void       Jitter(const Random&){ }        // Set random jitter factor
  Time_t     NextFreeTime() { return 0;}     // Get time link is free
  Count_t    NeighborCount(Node*);           // Number of routing neighbors
  void       Neighbors(NodeWeightVec_t&, Interface*, bool);
  void       AllNeighbors(NodeWeightVec_t&);    // Build neighbor list
  Count_t    PeerCount();                  // Return number of peers
  IPAddr_t   PeerIP(Count_t);              // Get peer IP Address (if known)
  IPAddr_t   NodePeerIP(Node*);            // Get peer IP Address (if known)
  bool       NodeIsPeer(Node*);            // Find out if node is a peer
  Count_t    NodePeerIndex(Node*);         // Get peer index for node
  IPAddr_t   DefaultPeer(Interface*);      // Get IP Addr of default peer
  DCount_t   BytesSent() { return 0;}
  DCount_t   PktsSent() { return 0;}
  bool       RxBroadcast() { return false;}// True if receive own bcast
private:
  Interface* pPeer;                        // Interface of single peer
public:
  static Count_t count;                    // Count of instances
};

#endif


