/*
* GTNetS provides a portable C++ class library for network simulations
*
* Copyright (C) 2005, The Boeing Company
*
* No guarantees or warranties or anything are provided or implied in any way
* whatsoever.  Use this program at your own risk.  Permission to use this
* program for any purpose is given, as long as the copyright is kept intact.
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

* Permission is hereby granted for any non-commercial use of this code
*/

// Georgia Tech Network Simulator - LinkMatrix connectivity

#ifndef __matrix_h__
#define __matrix_h__

#include "common-defs.h"
#include "node.h"
#include "link-real.h"
#include "ratetimeparse.h"
#include "l4protocol.h"
#include "interface-ethernet.h"
#include "ethernet.h"

typedef std::vector<Mult_t> MultVec;

class LinkMatrixEvent : public TimerEvent {

public:
  LinkMatrixEvent(Count_t s, Count_t d, Mult_t r);
  virtual ~LinkMatrixEvent();

  Count_t src_index;
  Count_t dst_index;
  Mult_t lossrate;
};

class LinkMatrix : public Ethernet, public TimerHandler {

public:
  LinkMatrix(Count_t,     // Node count
         IPAddr_t,    // IP addr start
         Mask_t,      // mask
         SystemId_t = 0,
         Rate_t r = Rate("10Mb"),
         Time_t dly = Time("1us"),
         Detail_t d = PARTIAL);

  virtual void Neighbors(NodeWeightVec_t&, Interface*, bool);
  
  virtual bool TransmitHelper(Packet*, Interface*, Node*, Interface*, 
                              bool bCast = false);
  Mult_t MatrixNeighborLossRate(Interface *src_if, Interface *dst_if);

  void SetNeighborLossRate(Count_t src_index, Count_t dst_index, 
                              Mult_t lossrate);

  void ScheduleMatrixChange(Time_t t, Count_t src_index,
                                      Count_t dst_index, Mult_t lossrate);

  virtual void Timeout(TimerEvent *pTimerEvent);
private:
  static Random* errRng;
  Timer timer;
  MultVec matrixVec;
};
#endif //__matrix_h__
