/*

* GTNetS provides a portable C++ class library for network simulations
*
* Copyright (C) 2003 George F. Riley
*
* No guarantees or warranties or anything are provided or implied in any way
* whatsoever.  Use this program at your own risk.  Permission to use this
* program for any purpose is given, as long as the copyright is kept intact.
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

* Permission is hereby granted for any non-commercial use of this code

*/

// Georgia Tech Network Simulator
/* 
   include the header files here for all layer 2 protocols 
   that llc encapsulation can recognize
*/
#include "llcsnap.h"
#include "ipv4.h"

// Specifies the size (in bytes) of the 802.3 header when transmitting
// on a link.   See comments in l2proto802.3.h for details.
Size_t LLCSNAPHeader::txSize = 8;

LLCSNAPHeader::LLCSNAPHeader(Word_t type)
{
  llc_dsap = 0xAA;
  llc_ssap = 0xAA;
  llc_ctrl = 0x3;
  snap_oui = 0;
  //bzero(&snap_oui, 2);
  snap_ethtype = type;
}

LLCSNAPHeader::LLCSNAPHeader(char* b, Size_t& sz, Packet* p)
{
  Size_t s = 0;
  b = Serializable::GetSize(b, sz, s);
  s  -= sizeof(Size_t);
  sz -= s;
  b  = Construct(b, s);
  p->PushPDUBottom(this);
  switch (snap_ethtype) { // Constuct appropriate l3pdu
  case 0x0800 : // IPV4
      new IPV4Header(b, sz, p); // Create and push IPV4 header
      break;
    }

}

char*  LLCSNAPHeader::Serialize(char* b, Size_t& sz)
{ // Serialize to a working buffer
  char dum = 0;
  b = SerializeToBuffer(b, sz, llc_dsap);
  b = SerializeToBuffer(b, sz, llc_ssap);
  b = SerializeToBuffer(b, sz, llc_ctrl);
  b = SerializeToBuffer(b, sz, dum); // Include an extra char for alignment
  b = SerializeToBuffer(b, sz, snap_oui);
  b = SerializeToBuffer(b, sz, snap_ethtype);
  return b;
}

char*  LLCSNAPHeader::Construct(char* b, Size_t& sz)
{
  char dum;
  
  b = ConstructFromBuffer(b, sz, llc_dsap);
  b = ConstructFromBuffer(b, sz, llc_ssap);
  b = ConstructFromBuffer(b, sz, llc_ctrl);
  b = ConstructFromBuffer(b, sz, dum);
  b = ConstructFromBuffer(b, sz, snap_oui);
  b = ConstructFromBuffer(b, sz, snap_ethtype);
  
  return b;
}
