/*

* GTNetS provides a portable C++ class library for network simulations
*
* Copyright (C) 2003 George F. Riley
*
* No guarantees or warranties or anything are provided or implied in any way
* whatsoever.  Use this program at your own risk.  Permission to use this
* program for any purpose is given, as long as the copyright is kept intact.
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

* Permission is hereby granted for any non-commercial use of this code

*/

// Georgia Tech Network Simulator - Manage IP Address Masks
// George F. Riley.  Georgia Tech, Spring 2002

#ifndef __mask_h__
#define __mask_h__

// Manage IP Address Masks
// Allows specification of masks as either a bit count or an actual
// mask value.

#include "common-defs.h"

//Doc:ClassXRef
class Mask {
public:
  typedef enum { MAX_MASK = 32 } MaxMask_t;
  Mask(Mask_t m = 24);  // Can be either a mask length or a mask (default 24)
  operator Mask_t() { return mask;} // Provide a typecast to a Mask_t
  Count_t  NBits() { return nBits;} // Number of bits
private:
  Mask_t mask;          // The actual mask
  Count_t nBits;        // Number of bits in the mask
};


#endif


