/*

* GTNetS provides a portable C++ class library for network simulations
*
* Copyright (C) 2003 George F. Riley
*
* No guarantees or warranties or anything are provided or implied in any way
* whatsoever.  Use this program at your own risk.  Permission to use this
* program for any purpose is given, as long as the copyright is kept intact.
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

* Permission is hereby granted for any non-commercial use of this code

*/

// Base class for re-using malloc'ed memory
// George F. Riley.  Georgia Tech, Winter 2002

#include <iostream>

#include "debug.h"
#include "common-defs.h"
#include "memreuse.h"

using namespace std;

// Static ReuseBase variables
PtrPtrVec_t   ReuseBase::q;          // A queue of deleted heap elements
unsigned long ReuseBase::reused;     // Statistics only
unsigned long ReuseBase::newalloc;
unsigned long ReuseBase::deleted;

void* ReuseBase::operator new(std::size_t s)
{
#ifdef NO_DEBUG
  // First compute size in words (32 bit units)
  register std::size_t sw = (s + sizeof(unsigned long) - 1) /
    sizeof(unsigned long);
  register void* b;
  if (q.size() <= sw || q[sw] == nil || q[sw]->size() == 0)
    { // No buffered heap elements of this size.  Allocate one and note lth
      // Allocate an extra 32 bit value for length field
      unsigned long* pl =(unsigned long*)malloc((sw+1)*sizeof(unsigned long));
      DEBUG0((cout << "Mallocing " << (sw+1) * sizeof(unsigned long) << endl));
      *pl = sw;
      b = &pl[1];  // Address to construct
      newalloc++;
    }
  else
    { // Reuse deleted element
      b = q[sw]->back();
      q[sw]->pop_back();
      reused++;
    }
  return ::operator new(s, b);// Call the in-place new
#else
  return ::operator new(s);   // In debug  mode, just use the normal "new"
#endif
}

void ReuseBase::operator delete(void* p)
{
#ifdef NO_DEBUG
  register std::size_t* pLth = (std::size_t*)p - 1;
  register std::size_t l = *pLth; // Size in 32 bit words
  while(q.size() <= l) q.push_back(nil); // Expand to correct size
  if (!q[l]) q[l] = new PtrVec_t;        // Allocate new vector if needed
  q[l]->push_back(p);                    // Add to saved queue
  deleted++;
#else
  ::delete (char*)p;                     // In debug mode, use normal delete
#endif
}

void ReuseBase::DumpQ()
{
  Count_t tot = 0;
  cout << "Printing HeapReuse information" << endl;
  cout << "Reused " << reused << " newalloc " << newalloc
       << " deleted " << deleted << endl;
  for (PtrPtrVec_t::size_type i = 0; i < q.size(); ++i)
    {
      cout << "Size " << i * sizeof(unsigned long);
      if (!q[i])
        {
          cout << " nil " << endl;
        }
      else
        {
          cout << " capacity " << q[i]->capacity() 
               << " size " << q[i]->size()
               << " total bytes " << q[i]->size() * i * sizeof(unsigned long)
               << endl;
          tot += q[i]->size() * i;
        }
    }
  cout << "Total bytes in reuse queue " << tot * sizeof(unsigned long) << endl;
}

