#include <iostream>
#include "mystat.h"
#include "simulator.h"

using namespace std;

MyStat::UIDVec_t MyStat::uidList;
ulong MyStat::pktsRequested;
ulong MyStat::pktsSent;
ulong MyStat::pktsReceived;
ulong MyStat::count[6];

MyStat::UIDMap_t MyStat::uidMap;
ulong MyStat::ctrlPkts[2];
ulong MyStat::ctrlBytes[2];
ulong MyStat::dataPkts[2];
ulong MyStat::dataBytes[2];
ulong MyStat::ctrlPktsUC;
ulong MyStat::ctrlPktsBC;

void MyStat::pktsReceive(ulong uid)
{
  // packet delivery fraction measure
  for (UIDVec_t::reverse_iterator i=uidList.rbegin(); i!=uidList.rend(); ++i)
    if (*i == uid) return;
  uidList.push_back(uid);
  pktsReceived++;

  // end-to-end delay measure
  UIDMap_t::iterator pos = uidMap.find(uid);
  pos->second.t2 = Simulator::Now();
}

void MyStat::pktsTimeStamp(ulong uid)
{
  uidMap.insert(UIDMap_t::value_type(uid, Times(Simulator::Now())));
}

void MyStat::Print()
{
  cout << "pktsRequested = " << pktsRequested << endl
       << "pktsSent      = " << pktsSent << endl
       << "pktsReceived  = " << pktsReceived << endl
       << "pktsLost      = " << pktsSent-pktsReceived << endl
       << "RQR           = " << (double)pktsReceived/pktsRequested << endl;
  if (pktsSent > 0)
    cout << "PDR           = " << (double)pktsReceived/pktsSent << endl;
  cout << "count1        = " << count[0] << endl
       << "count2        = " << count[1] << endl
       << "count3        = " << count[2] << endl
       << "count4        = " << count[3] << endl
       << "count5        = " << count[4] << endl
       << "count6        = " << count[5] << endl;

  // end-to-end delay
  double tot_delay=0;
  for(UIDMap_t::iterator i=uidMap.begin(); i!=uidMap.end(); ++i) {
    if (i->second.t2 == 0) continue;  // lost packet
    tot_delay += i->second.t2 - i->second.t1;
  }
  cout << "avg_delay     = " << tot_delay/pktsReceived << endl
       << "rt_load       = " << (double)ctrlPkts[0]/pktsReceived << endl;

  // routing overhead
  cout << "ctrlPkts      = " << ctrlPkts[0] << endl
       << "ctrlPktsUC    = " << ctrlPktsUC << endl
       << "ctrlPktsBC    = " << ctrlPktsBC << endl
       << "ctrlBytes     = " << ctrlBytes[0] << endl
       << "dataPkts      = " << dataPkts[0] << endl
       << "dataBytes     = " << dataBytes[0] << endl
       << "dataOverhead  = " << dataBytes[0] - 520*dataPkts[0] << endl
       << "ctrlPkts2     = " << ctrlPkts[1] << endl
       << "ctrlBytes2    = " << ctrlBytes[1] << endl
       << "dataPkts2     = " << dataPkts[1] << endl
       << "dataBytes2    = " << dataBytes[1] << endl
       << "dataOverhead2 = " << dataBytes[1] - 520*dataPkts[1] << endl;
}
