/*

* GTNetS provides a portable C++ class library for network simulations
*
* Copyright (C) 2003 George F. Riley
*
* No guarantees or warranties or anything are provided or implied in any way
* whatsoever.  Use this program at your own risk.  Permission to use this
* program for any purpose is given, as long as the copyright is kept intact.
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

* Permission is hereby granted for any non-commercial use of this code

*/

// Georgia Tech Network Simulator - PortMatching class
// Mohamed Abd Elhafez.  Georgia Tech, Spring 2004

// Implements the port matching algorithm for worm containment

#ifndef __port_matching_h__
#define __port_matching_h__

#include <queue>

#include "common-defs.h"
#include "pdu.h"
#include "ipaddr.h"
#include "ipv4.h"
#include "node.h"
#ifdef HAVE_QT
#include "qnamespace.h"
#endif
#include "tcp.h"
#include "worm-containment.h"

#define TCP_PROTOCOL 6
#define UDP_PROTOCOL 17


typedef std::vector<IPAddr_t> IPVec_t;
typedef std::vector<PortId_t> PortVec_t;
typedef std::map<PortId_t, Time_t> PortMap_t;
typedef std::pair<IPVec_t, double> IPNPair_t;
typedef std::map<PortId_t, IPNPair_t> SPortMap_t;


// Define the event subclass for DEWP  events
//Doc:ClassXRef
class DEWPEvent : public Event {
 public:
  typedef enum { PROCESS_QUEUE } DEWPEvent_t;
 public:
  DEWPEvent ()   { }
  DEWPEvent(Event_t ev)
    : Event(ev)
    { }
  virtual ~DEWPEvent() { }
 
};

//Doc:ClassXRef
class PortMatching : public WormContainment, public Handler{
//Doc:Class The class {\tt PortMatching} implements the port matching algorithm
//Doc:Class It derives 	from the class {\tt Handler}. 
	
public:
  //Doc:Method
  PortMatching();
  //Doc:Desc Default constructor

  // Handler
  virtual void Handle(Event*, Time_t);

  //Doc:Method
  void ProcessInPacket(Packet*, Interface*);
  //Doc:Desc Process the given packet for TCP SYN packets or UDP packets
  
 //Doc:Method
  void ProcessOutPacket(Packet*, IPAddr_t, int, Interface*);
  //Doc:Desc Process the given packet for TCP SYN packets or UDP packets
  
  void AddIP(SPortMap_t::iterator, IPAddr_t);

  bool IsInfected(PortId_t);
  void CheckInfected(SPortMap_t::iterator);



private:
  static Trace::TraceStatus traceStatus; // For tracing
  //Interface* iface;
  PortMap_t  incoming;
  PortMap_t  outgoing;
  SPortMap_t suspicious;
  PortVec_t  infected;
 
};

#endif

