#include "priqueue.h"
#include "ipv4.h"
#include "routing-nvr.h"
#include "l2proto802.11.h"

using namespace std;

bool PriQueue::Enque(Packet* p)
{
  int q = GetQueueType(p);
  return que[q].Enque(p);
}

Packet* PriQueue::Deque()
{
  for (int q = 0; q < NO_PRIQUEUE; q++) {
    Packet* p = que[q].Deque();
    if (p) return p;
  }
  return nil;
}

int PriQueue::GetQueueType(Packet* p)
{
  IPV4Header* iph = (IPV4Header*)p->PeekPDU(2);
  switch (iph->protocol)
    {
      /*
      case 0x8FF: {    // nvr
        NVROption* opt = (NVROption*)iph->options[0];
        if (opt->type == NVROption::NC_REP_SYM) return 0;
      }
      */
      case 0x8FF:    // nvr
      case 0x8FE:    // dsr
        	return 1;
      default:
        	return 2;
    }
}

Packet* PriQueue::Remove(MACAddr& dst)
{
  for (int q = 0; q < NO_PRIQUEUE; q++)
    for (PktList_t::iterator i=que[q].pkts.begin(); i!=que[q].pkts.end(); ++i)
      {
        DATA_frame* df = (DATA_frame*)(*i)->PeekPDU();
        if (!(df->addr1 == dst)) continue;
        Packet* p = *i;
        que[q].pkts.erase(i);
        que[q].size -= p->Size();
        que[q].pktCount--;
        return p;
      }
  return nil;
}

Count_t PriQueue::Length()
{
  Count_t len=0;
  for (int q = 0; q < NO_PRIQUEUE; q++)
    len += que[q].Length();
  return len;
}

Count_t PriQueue::LengthPkts()
{
  Count_t lenPkts=0;
  for (int q = 0; q < NO_PRIQUEUE; q++)
    lenPkts += que[q].LengthPkts();
  return lenPkts;
}

Queue* PriQueue::Copy() const
{
  return new PriQueue(*this);
}

void PriQueue::SetInterface(Interface* i)
{
  interface = i;
  for (int q = 0; q < NO_PRIQUEUE; q++)
    que[q].SetInterface(i);
}

void PriQueue::SetLimit(Count_t l)
{
  for (int q = 0; q < NO_PRIQUEUE; q++)
    que[q].SetLimit(l);
}

void PriQueue::SetLimitPkts(Count_t l)
{
  for (int q = 0; q < NO_PRIQUEUE; q++)
    que[q].SetLimitPkts(l);
}

bool PriQueue::Check(Size_t s, Packet* p)
{
  int q = GetQueueType(p);
  return que[q].Check(s);
}
