#include "propagation.h"
#include "common-defs.h"

double Propagation::defaultLoss = 1.0;
double Propagation::defaultGtx = 1.0;
double Propagation::defaultGrx = 1.0;
double Propagation::defaultPtx = 281.838293;       // 281.8 mW = 24.5 dBm
double Propagation::defaultFreq = 2400000000.0;    // 2.4 GHz

#ifdef SIM
double PropaTwoRay::defaultHtx = 1.5;
double PropaTwoRay::defaultHrx = 1.5;
#else
double PropaTwoRay::defaultHtx = 1.0;
double PropaTwoRay::defaultHrx = 1.0;
#endif


// Propagation classs

Propagation::Propagation()
{
  Loss = defaultLoss;
  Gtx = defaultGtx;
  Grx = defaultGrx;
  Ptx = defaultPtx;
  Freq = defaultFreq;
  Lambda = SPEED_LIGHT / Freq;   // lambda = c / f
  preFree = (Ptx*Gtx*Grx*Lambda*Lambda) / (16*PI*PI*Loss);
#ifdef SIM
	dSaved  = -1.0; // force update for the first call
	requiredSingalSaved = -1.0;// force update for the first call 
#endif //SIM
}

double Propagation::Pr(double d)
{
  // Friis free space propagation model
  double pr;

  /* For computation efficiency, use preFree.
  pr = (Ptx*Gtx*Grx*Lambda*Lambda) / (16*PI*PI*d*d*Loss);
  */
  pr = preFree / (d*d);
  return pr;
}


// PropaTwoRay class

PropaTwoRay::PropaTwoRay()
{
  Htx = defaultHtx;
  Hrx = defaultHrx;
  Crossover = (4*PI*Htx*Hrx) / Lambda;
  preTworay = (Ptx*Gtx*Grx*Htx*Htx*Hrx*Hrx) / Loss;
}

double PropaTwoRay::Pr(double d)
{
  if (d <= Crossover)
	return Propagation::Pr(d);
  else
	return preTworay / (d*d*d*d);
}

#ifdef SIM
void PropaTwoRay::SetPower(double d, double requiredSingal)
{
	if ( d == dSaved && requiredSingal == requiredSingalSaved) {
		return;
	}
	else { //update as necessary
	  dSaved = d;
		requiredSingalSaved = requiredSingal;
		if (d <= Crossover) 
			Ptx = requiredSingal * (16*PI*PI*d*d*Loss) / (Gtx*Grx*Lambda*Lambda)  ;
		else 
			Ptx = requiredSingal * (d*d*d*d*Loss)/ (Gtx*Grx*Htx*Htx*Hrx*Hrx) ;
			
		preFree = (Ptx*Gtx*Grx*Lambda*Lambda) / (16*PI*PI*Loss);
		preTworay = (Ptx*Gtx*Grx*Htx*Htx*Hrx*Hrx) / Loss;
	}
}
#endif //SIM
