/*

* GTNetS provides a portable C++ class library for network simulations
*
* Copyright (C) 2003 George F. Riley
*
* No guarantees or warranties or anything are provided or implied in any way
* whatsoever.  Use this program at your own risk.  Permission to use this
* program for any purpose is given, as long as the copyright is kept intact.
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

* Permission is hereby granted for any non-commercial use of this code

*/

// Georgia Tech Network Simulator - Protocol base class
// George F. Riley.  Georgia Tech, Spring 2002

// Class protocol is for any protocol stack layer implementation

#ifndef __protocol_h__
#define __protocol_h__

#include "common-defs.h"
#include "object.h"
#include "packet.h"
#include "trace.h"
#include "tfstream.h"

class Node;
class PDU;
//Doc:ClassXRef
class Protocol : public Object {
public:
  Protocol() : traceStatus(Trace::DEFAULT), traceDetails(ALLBITS)
    { }
  Protocol(Trace::TraceStatus ts) : traceStatus(ts), traceDetails(ALLBITS)
    { }
  virtual ~Protocol() { }
  virtual Layer_t Layer() = 0; // Determine which protocol layer
  // Packet tracing interface
  virtual void    SetTrace(Trace::TraceStatus ts)// Set trace level this proto
    { traceStatus = ts; }
  virtual Trace::TraceStatus GetTrace()          // Get trace level this proto
    { return traceStatus;}
  virtual void DetailOn(Bitmap_t b)  { traceDetails |= (1L << b);}
  virtual void DetailOff(Bitmap_t b) { traceDetails &= (ALLBITS ^ (1L << b));}
  virtual Bitmap_t Details() { return traceDetails;}
private:
  Trace::TraceStatus traceStatus;
  Bitmap_t           traceDetails;  // Bitmap of trace info desired
};

#endif

