/*

* GTNetS provides a portable C++ class library for network simulations
*
* Copyright (C) 2003 George F. Riley
*
* No guarantees or warranties or anything are provided or implied in any way
* whatsoever.  Use this program at your own risk.  Permission to use this
* program for any purpose is given, as long as the copyright is kept intact.
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

* Permission is hereby granted for any non-commercial use of this code

*/

// Georgia Tech Network Simulator - Protocol Graph class
// George F. Riley.  Georgia Tech, Spring 2002

// Defines the protocol graph for each node

#include <iostream>

#include "debug.h"
#include "protograph.h"

using namespace std;

ProtocolGraph* ProtocolGraph::common;

ProtocolGraph::ProtocolGraph() 
  : ProtoByLayer(N_LAYERS+1, ProtoMap_t())
{
}

Protocol* ProtocolGraph::Lookup(Layer_t layer, Proto_t proto)
{
  DEBUG0(
         cout << "ProtocolGraph::Lookup l" << layer 
         << " proto " << proto << endl);
  if (layer < 0 || layer > N_LAYERS) return NULL;
  DEBUG0(
         cout << "ProtocolGraph::Lookup returning " 
         << ProtoByLayer[layer][proto] << endl);
  return ProtoByLayer[layer][proto];
}

void ProtocolGraph::Insert(Layer_t layer, Proto_t proto, Protocol* p)
{
  ProtoByLayer[layer].insert(ProtoMap_t::value_type(proto, p));
}

void ProtocolGraph::Remove(Layer_t layer, Proto_t proto)
{
  ProtoMap_t& m = ProtoByLayer[layer];
  ProtoMap_t::iterator i = m.find(proto);
  if (i == m.end()) return; // Not found
  m.erase(i);               // Remove from map
}

// Static methods
void ProtocolGraph::CreateCommon()     // Create the shared graph if needed
{
  if (!common)
    common = new ProtocolGraph();
}

