/*

* GTNetS provides a portable C++ class library for network simulations
*
* Copyright (C) 2003 George F. Riley
*
* No guarantees or warranties or anything are provided or implied in any way
* whatsoever.  Use this program at your own risk.  Permission to use this
* program for any purpose is given, as long as the copyright is kept intact.
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

* Permission is hereby granted for any non-commercial use of this code

*/

// Georgia Tech Network Simulator - Packet/PDU Serializaton
// George F. Riley.  Georgia Tech, Spring 2002

#ifndef __serializable_h__
#define __serializable_h__

#include "common-defs.h"
#include "macaddr.h"

// Defines an abstract base class for serializable objects
// Serialization occurs in distributed simulations when packets
// must be moved from one system to another.

class Packet;
//Doc:ClassXRef
class Serializable {
public:
  virtual Size_t SSize() = 0;          // Size needed for serialization
  virtual char*  Serialize(char*, Size_t&) = 0;// Serialize to a buffer
  virtual char*  Construct(char*, Size_t&) = 0;// Construct from buffer

public:
  // Helpers
  char*  SerializeToBuffer(char*, Size_t&, size_t&);
#if defined (Linux) || defined (SunOS)
  char*  SerializeToBuffer(char*, Size_t&, unsigned long&);
#endif
  char*  SerializeToBuffer(char*, Size_t&, unsigned short&);
  char*  SerializeToBuffer(char*, Size_t&, float&);
  char*  SerializeToBuffer(char*, Size_t&, double&);
  char*  SerializeToBuffer(char*, Size_t&, MACAddr&);
  char*  SerializeToBuffer(char*, Size_t&, char&);

  char*  ConstructFromBuffer(char*, Size_t&, size_t&);
#if defined (Linux) || defined (SunOS)
  char*  ConstructFromBuffer(char*, Size_t&, unsigned long&);
#endif
  char*  ConstructFromBuffer(char*, Size_t&, unsigned short&);
  char*  ConstructFromBuffer(char*, Size_t&, float&);
  char*  ConstructFromBuffer(char*, Size_t&, double&);
  char*  ConstructFromBuffer(char*, Size_t&, MACAddr&);
  char*  ConstructFromBuffer(char*, Size_t&, char&);

  // Static methods
  static char* GetSize(char*, Size_t&, Size_t&);      // Get the size field
  static char* PutSize(char*, Size_t&, Size_t&);      // Put the size field
  static char* Dump(char*, Size_t);                   // Dump for debugging
  static void  Validate(Packet*);                     // Debug validation
  // Private methods
private:
  static void  CheckSize(Size_t&, Size_t);  // Check if sufficient size left
};

#endif


