/*

* GTNetS provides a portable C++ class library for network simulations
*
* Copyright (C) 2003 George F. Riley
*
* No guarantees or warranties or anything are provided or implied in any way
* whatsoever.  Use this program at your own risk.  Permission to use this
* program for any purpose is given, as long as the copyright is kept intact.
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

* Permission is hereby granted for any non-commercial use of this code

*/

// Georgia Tech Network Simulator - General Purpose Statistics base class
// George F. Riley.  Georgia Tech, Spring 2002

// Defines a general purpose statistics base class

#ifndef __statistics_h__
#define __statistics_h__

//Doc:ClassXRef
class Statistics {
  //Doc:Class Class {\tt Statistics} defines a virtual base class that
  //Doc:Class defines the behavior  of  all statistics collection objects
  //Doc:Class in \GTNS.
public:
  //Doc:Method
  virtual bool Record(double) = 0; // Record the specified value
    //Doc:Desc Record a new statistics sample.
    //Doc:Arg1 Sample to record.

  //Doc:Method
  virtual void Reset() = 0;        // Reset
    //Doc:Desc Remove all samples from the statistics collector and restore
    //Doc:Desc initial conditions.

  //Doc:Method
  virtual void Log(std::ostream&,
                   const char* hdr = nil,
                   const char sep = ',') = 0; // Log the recorded info
    //Doc:Desc Record the statistics on an output stream.
    //Doc:Arg1 Output stream to record  the  statistics on.
    //Doc:Arg2 Header line to add to beginning of the statistics log
    //Doc:Arg2 (if not {\tt nil}).
    //Doc:Arg3 Separator  character between statistics elements.

  virtual Statistics* Copy() const = 0;       // Make a copy of the statistics
};

#endif

