/*

* GTNetS provides a portable C++ class library for network simulations
*
* Copyright (C) 2003 George F. Riley
*
* No guarantees or warranties or anything are provided or implied in any way
* whatsoever.  Use this program at your own risk.  Permission to use this
* program for any purpose is given, as long as the copyright is kept intact.
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

* Permission is hereby granted for any non-commercial use of this code

*/

// Georgia Tech Network Simulator - TCP-Tahoe Class
// George F. Riley.  Georgia Tech, Spring 2002

// Implements the Tahoe version of TCP
// On any timeout, collapse congestion window to one segment
// Reduce slowstart threshold to 1/2 current cwnd, and resend
// the lowest sequence number in the retx list
//
// On any triple duplicate ack, resend the seq number requested,
// collapse cwnd to one segment, and reduce slowstart threshold
// to 1/2 the current cwnd.

#ifndef __tcp_tahoe_h__
#define __tcp_tahoe_h__

#include "tcp.h"
#include "node.h"

//Doc:ClassXRef
class TCPTahoe : public TCP {
  //Doc:Class Class {\tt TCPTahoe} defines the behavior of the {\em Tahoe}
  //Doc:Class variation of {\em TCP}.  The base class functions
  //Doc:Class {\tt DupAck} and {\tt ReTxTimeout} are overridden here, but
  //Doc:Class all other base class {\tt TCP} functions are used.

public:
  TCPTahoe();
  TCPTahoe(Node* n);
  TCPTahoe(const TCPTahoe&); // Copy constructor
  L4Protocol* Copy() const;  // Create a copy of this protocol
public:
  // Overridden TCP methods
  //Doc:Method
  virtual void DupAck(const TCPHeader&, Count_t); // Dup ack received
    //Doc:Desc Process a duplicate ack.
    //Doc:Arg1 TCP Header that caused the duplicate ack.
    //Doc:Arg2 Count of duplicates.

  //Doc:Method
  virtual void ReTxTimeout();   // Retransmit timeout
    //Doc:Desc Process a retransmit timeout.
};

#endif


