/*

* GTNetS provides a portable C++ class library for network simulations
*
* Copyright (C) 2003 George F. Riley
*
* No guarantees or warranties or anything are provided or implied in any way
* whatsoever.  Use this program at your own risk.  Permission to use this
* program for any purpose is given, as long as the copyright is kept intact.
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

* Permission is hereby granted for any non-commercial use of this code

*/

// Georgia Tech Network Simulator - TCP Port Demultiplexer
// George F. Riley.  Georgia Tech, Spring 2002

// Picks the TCP instance by port number

//#define DEBUG_MASK 0x03
#include "debug.h"
#include "tcpdemux.h"
#include "node.h"
#include "protograph.h"
#include "tcp.h"
#include "droppdu.h"
#include "icmp.h"

using namespace std;

// Define the pointer for the single tcpdemux instance
TCPDemux* TCPDemux::instance = nil;

// The common tcpDemux instance is defined in simulator.cc to insure 
// it's always loaded
// moved to here for testing
//TCPDemux     tcpDemux();

TCPDemux::TCPDemux()
{
  if (instance == NULL)
    { // First instance
      instance = this;
      ProtocolGraph::CreateCommon();
      ProtocolGraph::common->Insert(4, TCP::ProtoNumber, this); // Register
    }
}

#ifdef SIM
void TCPDemux::DataIndication(Node* n, Packet* p, IPAddr_t ip,
                              L3Protocol& l3, Interface* i, IPAddr_t dst)
#else
void TCPDemux::DataIndication(Node* n, Packet* p, IPAddr_t ip,
                              L3Protocol& l3, Interface* i)
#endif //SIM
{
  TCPHeader* pdu = (TCPHeader*)p->PeekPDU(); // Get the TCP pdu header (peek)
  TCP* tcp = (TCP*)n->LookupByPort(TCP::ProtoNumber,
                                   pdu->destPort, IPADDR_NONE,
                                   pdu->sourcePort, ip);
  if (!tcp)
    { // No port, discard and log
      DEBUG0((cout << "TCPDemux::DataIndication, no protocol at port " 
              << pdu->destPort 
              << " node " << (string)IPAddr(n->GetIPAddr())
              << endl));
      DropPDU d("L3-NP");
      n->TracePDU(&l3, &d);
      Stats::pktsDropped++;
      // Notify ICMP of port unreachable
      IPV4Header* iphdr = (IPV4Header*)p->PeekPDU(-1);
      ICMP::DestinationUnreachable(n, p, iphdr, ICMP::PORT_UNREACHABLE);
      delete p;
    }
  else
    {
      DEBUG(1,(cout << "TCPDemux found tcp " << tcp
               << " dp " << pdu->destPort
               << " sp " << pdu->sourcePort
               << " sip " << (string)IPAddr(ip)
               << endl));
      if (tcp->extraRxDelay != 0)
        {// Need to schedule an event for later
          Simulator::instance->Schedule(
              new L4DelayedRxEvent(tcp, n, p, ip, i),
              tcp->extraRxDelay, tcp);
        }
      else
        {
          tcp->DataIndication(n, p, ip, i); // Pass to the TCP instance
        }
    }
}

// Static methods
TCPDemux* TCPDemux::Instance()
{
  return instance;
}

void TCPDemux::Initialize()
{
  if (!instance) new TCPDemux();
}
