/*

* GTNetS provides a portable C++ class library for network simulations
*
* Copyright (C) 2003 George F. Riley
*
* No guarantees or warranties or anything are provided or implied in any way
* whatsoever.  Use this program at your own risk.  Permission to use this
* program for any purpose is given, as long as the copyright is kept intact.
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

* Permission is hereby granted for any non-commercial use of this code

*/

// Georgia Tech Network Simulator - Trace fstream object class
// George F. Riley.  Georgia Tech, Spring 2002

#ifndef __tfstream_h__
#define __tfstream_h__

#include <fstream>

#include "common-defs.h"
#include "ipaddr.h"

class Node;

// Tfstream is a trace stream; similar to fstream with some extensions
//Doc:ClassXRef
class Tfstream : public std::fstream {
public:
  Tfstream();
public:
  Tfstream& operator<<(char c);
  Tfstream& operator<<(unsigned char c) { return (*this) << (char)c; }
  Tfstream& operator<<(signed char c) { return (*this) << (char)c; }
  Tfstream& operator<<(const char *s);
  Tfstream& operator<<(const unsigned char *s)
    { return (*this) << (const char*)s; }
  Tfstream& operator<<(const signed char *s)
    { return (*this) << (const char*)s; }
  Tfstream& operator<<(const void *p);
  Tfstream& operator<<(int n);
  Tfstream& operator<<(unsigned int n);
  Tfstream& operator<<(long n);
  Tfstream& operator<<(unsigned long n);
  Tfstream& operator<<(short n) {return operator<<((int)n);}
  Tfstream& operator<<(unsigned short n) {return operator<<((unsigned int)n);}
  Tfstream& operator<<(bool b) { return operator<<((int)b); }
  Tfstream& operator<<(double n);
  Tfstream& operator<<(float n) { return operator<<((double)n); }
  Tfstream& operator<<(IPAddr& i);
public:
  bool AppendEOL();      // Append eol to current line if needed
  void NewNode(Node* n); // Log the node for protocol logging
  void SetNode(Node* n); // Set node pointer for subsequent traces
  void IPDotted(bool d) { ipDotted = d;}
  void TimePrecision(Count_t); // Set new precision
public:
  bool    pendingEOL;    // True if EndOfLine needed
private:
  Count_t timePrecision; // Number significant digits on time in trace file
  Node* node;
  bool  ipDotted;        // True if trace IP Addresses with dotted notation
  Time_t digitsThreshold;// Time that sprintf format must change
  std::string format;    // Holds the current time format
private:
  inline void CheckLogTime()
    {
      if (!pendingEOL && is_open())
        {
          LogSimTime();
        }
    }
  void LogSimTime();
};

#endif
