/*

* GTNetS provides a portable C++ class library for network simulations
*
* Copyright (C) 2003 George F. Riley
*
* No guarantees or warranties or anything are provided or implied in any way
* whatsoever.  Use this program at your own risk.  Permission to use this
* program for any purpose is given, as long as the copyright is kept intact.
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

* Permission is hereby granted for any non-commercial use of this code

*/


//TransitNode Class Definitions
//Talal Jaafar, Georgia Tech, Spring 2003

#ifndef __TRANSITNODE_H__
#define __TRANSITNODE_H__

#include "node.h"
#include "subdomain.h"

typedef std::vector<Node*>  NodeList_t;
typedef std::vector<SubDomain*> SubList_t;

//Doc:ClassXRef
class TransitNode {
  //Doc:Class Defines an interface for the transit node

  Node *Tnode;            //transit Node
  SubList_t Slist;        //a list of the subdomains associated with this transit node 
  int MaxSubId;           //the highest Sub id in the SList (above)
 public:
  //Doc:Method
  TransitNode(int nodeNum, int x, int y) {
    Tnode = new Node(nodeNum);
    Tnode->SetLocation(x,y);
  }
  //Doc:Desc This method is the default constructor (set node id, X & Y coordinates)
  //Doc:Arg1 The node number (id)
  //Doc:Arg2 The x coordinate of the node
  //Doc:Arg3 The y coordinate of the node

  //Doc:Method
  void AddSub(SubDomain *Sub) {
    Slist.push_back(Sub);
    MaxSubId = Sub->GetSubId();
  }
  //Doc:Desc Associate a new Sub with this transit node
  //Doc:Arg1 Pointer to a subdomain

  //Doc:Method
  Node* GetTNode() { return Tnode; }
  //Doc:Desc Return the transit node
  //Doc:Return Pointer to the transit node

  //Doc:Method
  SubList_t GetSubList() { return Slist;}
  //Doc:Desc Return the subs associated with this transit node
  //Doc:Return A vector of subdomains

  //Doc:Method
  void SetSubList(SubList_t subList) { Slist = subList; }
  //Doc:Desc Set the Sub list to the passed list
  //Doc:Arg1 A vector of subdomains

  //Doc:Method
  int GetMaxSubId() { return MaxSubId; }
  //Doc:Desc This method is used to return the highest Sub id associated with this transit node
  //Doc:Return An integer

  //Doc:Method
  int GetId() { return Tnode->Id(); }
  //Doc:Desc This method is used to return the transit node id
  //Doc:Return An integer

  //return the IPAddress
  //Doc:Method
  IPAddr_t GetIP() { return Tnode->GetIPAddr(); }
  //Doc:Desc This method is used to return the IP Address of the transit node
  //Doc:Return The IP address of the transit node

};

#endif
