/*

* GTNetS provides a portable C++ class library for network simulations
*
* Copyright (C) 2003 George F. Riley
*
* No guarantees or warranties or anything are provided or implied in any way
* whatsoever.  Use this program at your own risk.  Permission to use this
* program for any purpose is given, as long as the copyright is kept intact.
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

* Permission is hereby granted for any non-commercial use of this code

*/

// Georgia Tech Network Simulator - Grid Network Object
// George F. Riley.  Georgia Tech, Spring 2002

// Define a two dimensional grid of wireless nodes

#ifndef __wireless_grid_recatngular_h__
#define __wireless_grid_rectangular_h__

#include "wireless-grid.h"

//Doc:ClassXRef
class WirelessGridRectangular : public WirelessGrid {
public:
  WirelessGridRectangular(const Location&,   // Lower left
                          const Random&,     // Size
                          const Random&,     // x/y Random 
                          IPAddr_t = IPADDR_NONE, // Starting ip
                          bool = true);      // Create nodes
  WirelessGridRectangular(const Location&,   // Lower left
                          const Random&,     // Size
                          const Random&,     // x random
                          const Random&,     // y random
                          IPAddr_t = IPADDR_NONE, // Starting ip
                          bool = true);      // Create nodes
#ifdef MANET
  WirelessGridRectangular(L2ProtocolType, // mac protocol
                          const Location&,   // Lower left
                          const Random&,     // Size
                          const Random&,     // x/y Random 
                          IPAddr_t = IPADDR_NONE, // Starting ip
                          bool = true);      // Create nodes
  WirelessGridRectangular(L2ProtocolType, // mac protocol
                          const Location&,   // Lower left
                          const Random&,     // Size
                          const Random&,     // x random
                          const Random&,     // y random
                          IPAddr_t = IPADDR_NONE, // Starting ip
                          bool = true);      // Create nodes
#endif //MANET
  ~WirelessGridRectangular();
  Location GetRandomLocation();
private:
  void    CreateNodes();    // Create the nodes
private:
  Location lowerLeft;
  Random*  sizeRng;
  Random*  xLocationRng;
  Random*  yLocationRng;
#ifdef MANET
  L2ProtocolType l2;
#endif // MANET
};

#endif
