/*

* GTNetS provides a portable C++ class library for network simulations
*
* Copyright (C) 2003 George F. Riley
*
* No guarantees or warranties or anything are provided or implied in any way
* whatsoever.  Use this program at your own risk.  Permission to use this
* program for any purpose is given, as long as the copyright is kept intact.
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

* Permission is hereby granted for any non-commercial use of this code

*/

// Georgia Tech Network Simulator - Base Class for Wireless Grid Object
// George F. Riley.  Georgia Tech, Spring 2002

// Base class for  a two dimensional grid

#ifndef __wireless_grid_h__
#define __wireless_grid_h__

#include "common-defs.h"
#include "rng.h"
#include "node.h"

class Mobility;

//Doc:ClassXRef
class WirelessGrid {
public:
  WirelessGrid(IPAddr_t i) : first(Node::nextId), startingIP(i), size(0) { }
  virtual ~WirelessGrid() { }
  virtual Count_t Size() { return size;}
  // Get a pointer to the i'th node      
  virtual Node*    GetNode(Count_t i);
  void    AddMobility(const Mobility&); // Add a mobility model to every node
#ifdef SIM
  void AddMobility(const Mobility&, Node*); //Add mobility to a single node
#endif //SIM
  virtual Location GetRandomLocation() = 0;
public:
  NodeId_t first;
  IPAddr_t startingIP;
  Count_t  size;
};

#endif
