/*

* GTNetS provides a portable C++ class library for network simulations
*
* Copyright (C) 2003 George F. Riley
*
* No guarantees or warranties or anything are provided or implied in any way
* whatsoever.  Use this program at your own risk.  Permission to use this
* program for any purpose is given, as long as the copyright is kept intact.
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

* Permission is hereby granted for any non-commercial use of this code

*/

// Georgia Tech Network Simulator - WormContainment class
// Mohamed Abd Elhafez.  Georgia Tech, Summer 2004

#ifndef __worm_containment_h__
#define __worm_containment_h__

#include "packet.h"
#include "interface.h"
#include "ipaddr.h"


//Doc:ClassXRef
class WormContainment {
  //Doc:Class Objects of class {\tt WormContainment} are used in \GTNS\ as a base class
  //Doc:Class for all worm containment algorithms

public:
  //Doc:Method
  virtual void ProcessInPacket(Packet* p, Interface* iface)
    {
      iface->GetL2Proto()->DataIndication(p);
    }
  //Doc:Desc Process the incoming packet
  //Doc:Arg1 The packet received
  //Doc:Arg2 The Receiving interface

 //Doc:Method
  virtual void ProcessOutPacket(Packet* p, IPAddr_t i, int type, Interface* iface)
    {
      iface->Xmit(p, i, type);
    }
  //Doc:Desc Process the outgoing packet
  //Doc:Arg1 The packet to be sent
  //Doc:Arg2 Ip address of the next hop
  //Doc:Arg3 Type of the packet
  //Doc:Arg4 The sending interface
};


#include "interface.h"

#endif
