/*

* GTNetS provides a portable C++ class library for network simulations
*
* Copyright (C) 2003 George F. Riley
*
* No guarantees or warranties or anything are provided or implied in any way
* whatsoever.  Use this program at your own risk.  Permission to use this
* program for any purpose is given, as long as the copyright is kept intact.
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

* Permission is hereby granted for any non-commercial use of this code

*/

// Georgia Tech Network Simulator - Worm Application class

// Monirul I Sharif.  Georgia Tech, Spring 2004

// Define a Worm Application class

#ifndef __worm__h__
#define __worm__h__

#include "common-defs.h"
#include "application.h"
#include "timer.h"
#include "tcp.h"
#include "rng.h"
#include "wormtargetvector.h"
#include <string>

#include <map>


//Doc:ClassXRef
class Worm : public Application {
  //Doc:Class Define a application that works as the Sapphire worm
  //Doc:Class 
public:
  // Define a number of constructors for different needs
  //Doc:Method
  Worm();
    //Doc:Desc Specifies a node to attach the worm application to. 

  virtual void AttachNode(Node*); // Note which node attached to
  virtual ~Worm() {};
  virtual Application* Copy() const;// Make a copy of the application
  

  // newly defined in this class
  virtual void Initialize();

  void Infect();
  bool IsInfected();

  Node * AttachedNode();

  static void SetPayloadLength(Size_t);
  static void SetBaseIP(IPAddr_t );
  static void SetScanRange(Count_t);

  static void SetVulnerability(double);
  static void SetInfectionPort(PortId_t);
  static void SetTargetVector(const WormTargetVector &);
  static void SetSignature(std::string s);

  static Count_t  TotalInfected() {return totalinfected;};
  static Count_t  TotalVulnerable() {return totalvulnerable;};
  static Count_t  TotalInstances() {return totalinstances;};


protected:
  bool		infected;      // indicates if this node is in infected stage
  bool          vulnerable;
  bool          started;       // used to know if the application of worm has started

  IPAddr_t GenerateNextIPAddress();
  bool PacketIsWorm(Packet *);   // I am going to use this to check if packet 
                                 // is worm
  virtual void PrepareWormData(char *&);

  virtual void Activate();
  virtual void StartApp();    // Called at time specified by Start
  virtual void StopApp();     // Called at time specified by Stop

  Node*		node;

  static Size_t payloadlength;
  static PortId_t infectionport;
  static double vulnerability;
  static IPAddr_t baseIP;
  static std::string signature;    // signature string that is sent in packets
  static char* wormdata;           // the contents of the worm  (this is sent usually sent)

  virtual void Receive(Packet *, L4Protocol *, Seq_t); // Data received
  virtual void Sent(Count_t , L4Protocol *); 
  virtual void ConnectionComplete(L4Protocol *);
  virtual void ConnectionFailed(L4Protocol *);
  virtual void CloseRequest(L4Protocol *);
  virtual void Closed(L4Protocol *);

 private:
  static Count_t  totalinfected;    // keeps count of total infected instances
  static Count_t  totalvulnerable;  // keeps count of total vulnerable instances
  static Count_t  totalinstances;

  WormTargetVector *targetV;
  static WormTargetVector *defaultTV;

  static bool initialized;  // set to see if worm class was initialized
  static Random* rngV;

};

#endif
