/*

* GTNetS provides a portable C++ class library for network simulations
*
* Copyright (C) 2003 George F. Riley
*
* No guarantees or warranties or anything are provided or implied in any way
* whatsoever.  Use this program at your own risk.  Permission to use this
* program for any purpose is given, as long as the copyright is kept intact.
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

* Permission is hereby granted for any non-commercial use of this code

*/


// Georgia Tech Network Simulator - Worm Application

// Monirul I Sharif,  Georgia Tech, Fall 2003
// This is the TCP version test

// Define a Worm Application class

// Uncomment below to enable debug level 0

//#define DEBUG_MASK 0x1

#include "debug.h"
#include "wormtcp.h"
#include "tcp-tahoe.h"

#include "node.h"
#include "simulator.h"
#include <map>

#ifdef HAVE_QT
#include <qnamespace.h>
#endif

using namespace std;


Random*  WormTCP::rngD = nil;
Count_t  WormTCP::connections = 2;

// In this version of the worm application I am using a constant
//    packets per second method to sent UDP packets.
// I will later change this into a more robust method.

// Constructors


WormTCP::WormTCP()
    : tcp(nil)
{
  //  DEBUG0((cout << "Attaching application " << this << endl));

  tcp_c.clear();
  tcp_c_map.clear();
  tcp_r_count.clear();
  sentAck.clear();

  if (!rngD) rngD = new Uniform(0, 1);
}


// Application Methods
void WormTCP::StartApp()     // Called at time specified by Start
{
  Worm:: StartApp();
}

void WormTCP::StopApp()    // Called at time specified by Stop
{
  Worm :: StopApp();
}

void WormTCP::AttachNode(Node* n)
{
  Worm::AttachNode(n) ;

  tcp = new TCPTahoe(n);
  
  tcp->Bind(infectionport);
  tcp->Listen();
  tcp->AttachApplication(this);

  DEBUG(1,(cout <<"tcp   : " << tcp <<", attached application "<< tcp->GetApplication()<<endl));

  for (int i =0; i<(int)connections; i++) {
    TCP * temp_tcp = new TCPTahoe(n);
    
    temp_tcp->AttachApplication(this);
    tcp_c.push_back(temp_tcp);
    sentAck.push_back(0);
    tcp_c_map[temp_tcp]=i;
    DEBUG(1,(cout <<"tcp_c : " << tcp_c[i] <<", attached application "<< tcp_c[i]->GetApplication()<<endl));  
  }
}

Application* WormTCP::Copy() const
{
  return new WormTCP(*this);
}

void WormTCP::Activate()
{
  DEBUG0(( cout<<"StartInfection["<<(string)IPAddr(node->GetIPAddr())<<
	   "]: Infected machine."<<endl ));
  // Set the node to red on animation
#ifdef HAVE_QT
  node->Color(Qt::red);
#endif

  for (int i =0; i<(int) connections; i++) 
    SendWorm(i);
}

void WormTCP::Initialize()
{
  cout<<"TCP initialize"<<endl;
  // override this in child classes
}


void WormTCP::SendWorm(int threadno)
{
  //  char buffer[payloadlength+1];   // later on I'll add the real contents here
  
  IPAddr_t target = GenerateNextIPAddress();  
  
  DEBUG0((cout<<"SendWorm["<<(string)IPAddr(node->GetIPAddr())<<
	  "]: <Thread "<<threadno<<"> first connecting to "<<(string)IPAddr(target)<<endl));

  tcp_c[threadno]->Connect(target, infectionport);
  
  //  udp->SendTo(payloadlength, target, defaultinfectionport);
  //  DEBUG0((cout<<"sent worm packet"<<endl));  
}

void WormTCP::ConnectionComplete(L4Protocol *proto)
{
  int threadno = tcp_c_map[(TCP*)proto];
  
  DEBUG0((cout<<"ConnectionComplete["<<(string)IPAddr(node->GetIPAddr())<<
	  "]: <Thread "<<threadno<<"> connection successful to "<<(string)IPAddr(proto->peerIP)
	  <<" now sending packet of length "<<payloadlength<<endl));

 
  sentAck[threadno]=0;

  // a small hack to add some transmission delay sometimes
  /*
  double rvalue = rngD->Value();
  if (rvalue>0.5)
    rvalue = 0;
  tcp_c[threadno]->AddExtraTxDelay(rvalue / 1000);
  */

  tcp_c[threadno]->Send(wormdata, payloadlength);
  DEBUG(1,(cout<<"Contents:"<<wormdata<<endl));
}

void WormTCP::Sent(Count_t c, L4Protocol *proto)
{
  int threadno = tcp_c_map[(TCP*)proto];

  sentAck[threadno]+=c;

  if (sentAck[threadno]>=payloadlength) {
    DEBUG0((cout <<"Sent["<<(string)IPAddr(node->GetIPAddr())<<"]: <Thread "<<threadno<<"> All worm data sent, now closing"<<endl));
    tcp_c[threadno]->Close();
  }
}

void WormTCP::ConnectionFailed(L4Protocol *proto)
{
  int threadno = tcp_c_map[(TCP*)proto];

  DEBUG0((cout<<"ConnectionFailed["<<(string)IPAddr(node->GetIPAddr())<<
	  "]: <Thread "<<threadno<<"> Connection failed to "<<(string)IPAddr(proto->peerIP)<<endl));
  SendWorm(threadno);
}

void WormTCP::CloseRequest(L4Protocol *proto)
{
  DEBUG0((cout<<"CloseRequest["<<(string)IPAddr(node->GetIPAddr())<<"]: Close request received, now closing"<<endl));
  tcp_r_count.erase((TCP*)proto);
  tcp_r_isworm.erase((TCP*)proto);

  proto->Close();
}

void WormTCP::Closed(L4Protocol *proto)
{
  int threadno = tcp_c_map[(TCP*)proto];

  DEBUG0((cout<<"Closed["<<(string)IPAddr(node->GetIPAddr())<<"]:thread "<<threadno<<" connection has closed."<<endl));
  // I am doing a hack here...
  // I am creating a new TCP object and pointing it using tcp_c... I think I should reuse this one
  // but for now I can't
  
  tcp_c_map.erase((TCP*)proto);

  TCP *temp_tcp = new TCPTahoe(node);
  temp_tcp->AttachApplication(this);
  tcp_c_map[temp_tcp] = threadno;
  tcp_c[threadno]=temp_tcp;

  DEBUG0((cout <<"tcp_c : " << temp_tcp <<", attached application "<< temp_tcp->GetApplication()<<endl));
  
  SendWorm(threadno);

}

void WormTCP::SetConnections(Count_t th)
{
  connections = th;
}

void WormTCP::Receive(Packet *p, L4Protocol *proto, Seq_t)
{
  Count_t received = tcp_r_count[(TCP*)proto];

  DEBUG0((cout<<"Receive["<<(string)IPAddr(node->GetIPAddr())<<"]: Received TCP packet of size "<<p->Size()<<
	  "Prev received "<<received<<endl));
  
  if (received==0) {
    tcp_r_isworm[(TCP*)proto]= PacketIsWorm(p);
  }
  received+=p->Size();
  tcp_r_count[(TCP*)proto] = received;

  if (tcp_r_isworm[(TCP*)proto] && received>=payloadlength) {
    if (vulnerable && !infected) {
      Infect();
    }
  }

  delete p;
}
