/*

* GTNetS provides a portable C++ class library for network simulations
*
* Copyright (C) 2003 George F. Riley
*
* No guarantees or warranties or anything are provided or implied in any way
* whatsoever.  Use this program at your own risk.  Permission to use this
* program for any purpose is given, as long as the copyright is kept intact.
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

* Permission is hereby granted for any non-commercial use of this code

*/

// Georgia Tech Network Simulator - WormTCP Application class

// Monirul I Sharif.  Georgia Tech, Spring 2004

// Define a WormTCP Application class

#ifndef __wormtcp__h__
#define __wormtcp__h__

#include "common-defs.h"
#include "application.h"
#include "timer.h"
#include "tcp.h"
#include "rng.h"
#include "worm.h"

#include <map>

// This version only supports TCP protocol

//Doc:ClassXRef
class WormTCP : public Worm {
  //Doc:Class Define a application that works as the Sapphire worm
  //Doc:Class 
public:
  // Define a number of constructors for different needs
  //Doc:Method
  WormTCP();   // Just specify endpoint
    //Doc:Desc Specifies a node to attach the worm application to. 

  virtual void AttachNode(Node*); // Note which node attached to
  virtual ~WormTCP() {};
  virtual Application* Copy() const;// Make a copy of the application
  
  static void SetConnections(Count_t);

  virtual void Initialize();

public:
  // newly defined in this class
  virtual void Activate();

private:
  TCP*                  tcp;           // this one is used for the server side
  std::vector <TCP*>    tcp_c;         // the client side protocols
  std::map <TCP*,int>   tcp_c_map;     // client side protocol mapping to know how much data ack'd
  std::map <TCP*,Count_t>   tcp_r_count;     // server side protocol mapping to know how much data has been received
  std::map <TCP*,bool>  tcp_r_isworm;    // server side protocol mapping to know store if packets that are coming is a worm

  std::vector <Count_t> sentAck;

  static Random* rngD;   // for adding some transmission delay

  static Count_t connections;

  void SendWorm(int);

  // the following method is called when packets are received from the port
  virtual void Receive(Packet *, L4Protocol *, Seq_t); // Data received
  virtual void Sent(Count_t , L4Protocol *); 
  virtual void ConnectionComplete(L4Protocol *);
  virtual void ConnectionFailed(L4Protocol *);
  virtual void CloseRequest(L4Protocol *);
  virtual void Closed(L4Protocol *);

  virtual void StartApp();    // Called at time specified by Start
  virtual void StopApp();     // Called at time specified by Stop
};

#endif

