# comp_density_or.pl:  This script runs random_waypoint_manet with Overlapping
# Relays (OR)s and varies the radio range and number of nodes.
# The results provide information on the performance of ORs with varying
# density.
#
# Below is a list of parameters that can be varied.  Each run of the scipt
# will loop over seeds, radio range, and the number of nodes in the MANET.
#
# The script will generate *.log and *.stat files.  The log files contain
# the GTNetS simulation progress.  The stat file gives the statistics
# generated by the particular simulation.
#
# The environment variable GTNETS_HOME must be set to the base GTNetS directory.
#

eval 'exec perl -S $0 ${1+"$@"}'  # -*-Mode:Perl; perl-indent-level:8-*-
if 0;
use POSIX;
use Env;  #Environment Variables

$dir_rwp =  $GTNETS_HOME."/ietf/random_waypoint_manet";
$test="random_waypoint_manet";

#Basic config
@seed = (2); #(1,2,3,4); #1;
$start_time=1800;
$stop_time=2700;
# no trace, etrace, osfpv3log, tag=args

#Data traffic config
$pktsize=40;
$pktrate=10;

#Scenario config
@num_nodes = (10,20,30,40,50);
@radio_range = (100,120,140,160,180,200,220,240,260,280,300); 
$alpha = 0.5;
$grid_length = 500;
$pause_time =  1;
$velocity = 16; #10;

#Mac config
$mac_protocol = 0;

#OSPFv3 config
$wireless_interface=2; # ptmp=1, manet=2
$wireless_flooding= 1; # broadcast=0, mpr= 1, sicds = 2
$HelloInterval = 2;
$DeadInterval = 6;
$RxmtInterval = 5;
$MinLSInterval = 5;
$MinLSArrival = 1;

#MPR MANET config
$PushbackInterval = 3500; #default 2000 msec
                          #PushbackInterval + propagation delay) <
                          #RxmtInterval/2
                          #Range in GTNetS 3.5 to 7 sec
$AckInterval = 500;    #default 1800 msec
                       #(AckInterval + propagation delay) < PushbackInterval
                       #Range in GTNetS 0 to 0.5 sec
$smart_peering = "SmartPeering";
$unsynch_adj = "UnsynchAdj";
$diff_hellos = ""; #"diff_hellos";

foreach $seed(@seed) {
  foreach $num_nodes(@num_nodes) {
    foreach $radio_range(@radio_range) {

      $tag = "MPR.SP.radio_range".$radio_range."seed".$seed;
      $cmd_input = " seed=".$seed." num_nodes=".$num_nodes.
             " tag=".$tag.".num_nodes=".$num_nodes.
             " pktrate=".$pktrate.
             " wireless_interface=".$wireless_interface.
             " wireless_flooding=".$wireless_flooding.
             " velocity=".$velocity.
             " radio_range=".$radio_range.
             " pause_time=".$pause_time.
             " HelloInterval=".$HelloInterval.
             " RxmtInterval=".$RxmtInterval.
             " DeadInterval=".$DeadInterval.
             " PushbackInterval=".$PushbackInterval.
             " ".$smart_peering.
             " ".$unsynch_adj.
             " AckInterval=".$AckInterval.
             " ".$diff_hellos.
             " start_time=".$start_time." stop_time=".$stop_time;

      system("$dir_rwp/$test-opt $cmd_input > $dir_rwp/$test-opt-$tag.log");
    }
  }
}
