# comp_mdr_or-sp.pl:  This script runs random_waypoint_manet and compares:
# 1. MDRs with uniconnected adjacencies and full LSAs
# 2. MDRs with biconnected adjacencies and mincost LSAs
# 3. ORs with smart peering and full LSAs.
#
# Below is a list of parameters that can be varied.  MANET parameters can
# be found in the "ManetScenario" subroutine at the bottom of the code.  
# Each run of the scipt will loop over seeds, radio range, 
# pause time, velocity, alpha, and the number of nodes in the MANET.
#
# The script will generate *.log and *.stat files.  The log files contain
# the GTNetS simulation progress.  The stat file gives the statistics
# generated by the particular simulation.  The stat files will be placed
# in a directory call comp_mdr_or-sp
#
# After all simulations have been performed the script will average results
# with the same seed and coalesce some of the data into a file called 
# "data.dat".
#
# The environment variable GTNETS_HOME must be set to the base GTNetS directory.

eval 'exec perl -S $0 ${1+"$@"}'  # -*-Mode:Perl; perl-indent-level:8-*-
if 0;
use POSIX;
use Env;  #Environment Variables


$dir_rwp =  $GTNETS_HOME."/ietf/random_waypoint_manet";
$dir_output = $dir_rwp."/comp_mdr_or-sp";
system("mkdir -p $dir_output");
$test="random_waypoint_manet";

#Basic config
@seed = (2); #(1,2,3,4); #1;
$start_time=1800;
$stop_time=3000;
# no trace, etrace, osfpv3log, tag=args

#Data traffic config
$pktsize=40;
$pktrate=10;

#Scenario config
@num_nodes = (10,30,50,70); #(10,25,50,75);
@radio_range = (100,140,180,220,260,300,1000);
@pause_time = (1, 40);
@velocity = (0,5,10,20);
@alpha = (0.5); #(0.5,1);
$grid_length = 500;

#Mac config
$mac_protocol = 0;

#OSPFv3 config
$wireless_interface=2; # ptmp=1, manet=2
# See subroutine ManetScenario for OSPF parameters

@manet_scenario = ("ORSPfull","MDRunifull","MDRbimincost");

if(1) {
foreach $seed(@seed) {
  foreach $num_nodes(@num_nodes) {
    foreach $radio_range(@radio_range) {
      foreach $pause_time(@pause_time) {
        foreach $velocity(@velocity) {
          foreach $alpha(@alpha) {
            foreach $manet_scenario(@manet_scenario) {

  ManetScenario($manet_scenario);
  $tag = $wf.".".$ac.".".$lf.".seed=".$seed.".rr=".$radio_range.".pt=".$pause_time.".vel=".$velocity.".alpha=".$alpha.".nn=".$num_nodes;

  $cmd_input = " seed=".$seed.
               " num_nodes=".$num_nodes.
               " tag=".$tag.
               " pktrate=".$pktrate.
               " wireless_interface=".$wireless_interface.
               " wireless_flooding=".$wireless_flooding.
               " velocity=".$velocity.
               " radio_range=".$radio_range.
               " alpha=".$alpha.
               " pause_time=".$pause_time.
               " HelloInterval=".$HelloInterval.
               " RxmtInterval=".$RxmtInterval.
               " DeadInterval=".$DeadInterval.
               " AckInterval=".$AckInterval.
               " ".$diff_hellos.
               " PushbackInterval=".$PushbackInterval.
               " ".$smart_peering.
               " ".$unsych_adj.
               " BackupWaitInterval=".$BackupWaitInterval.
               " TwoHopRefresh=".$TwoHopRefresh.
               " AdjConnectivity=".$AdjConnectivity.
               " LSAFullness=".$LSAFullness. 
               " start_time=".$start_time.
               " stop_time=".$stop_time;

      system("$dir_rwp/$test-opt $cmd_input > $dir_rwp/$test-opt.$tag.log");
      system("mv $dir_rwp/$test.$tag.stat $dir_output");

            }
          }
        }
      }
    }
  }
}
}

$outputfile = "data.dat";
system ("rm $outputfile");
foreach $num_nodes(@num_nodes) {
  foreach $radio_range(@radio_range) {
    foreach $pause_time(@pause_time) {
      foreach $velocity(@velocity) {
        foreach $alpha(@alpha) {
          foreach $manet_scenario(@manet_scenario) {

  ManetScenario($manet_scenario);
  $tag = $wf.".".$ac.".".$lf.".seed=*.rr=".$radio_range.".pt=".$pause_time.".vel=".$velocity.".alpha=".$alpha.".nn=".$num_nodes;
  $file = $test.".".$wf.".".$ac.".".$lf.".rr=".$radio_range.".pt=".$pause_time.".vel=".$velocity.".alpha=".$alpha.".nn=".$num_nodes.".dat";

  system("cd $dir_output; $GTNETS_HOME/bin/statistics.pl -both *$tag.stat; mv mean.out mean_$file");

  @selected = (79,90,96,124); #overhead, adjpernode,lsaorig,delivratio 
  open(MEAN, "< $dir_output/mean_".$file) or die "Error: mean_$file, $!";
  open(OUT, ">> ".$outputfile) or die "Error: $outputfile, $!";
  printf(OUT "%s %s %d %d %d %f %d ", 
    $ac, $lf, $radio_range, $pause_time, $velocity, $alpha, $num_nodes);

  $index = 0;
  <MEAN>; #Read top line of file, discard
  while(<MEAN>) {
    @fields = /([\d\.]+)/g;
    while (scalar(@fields)) {
      $current = shift @fields;
      if ($index == $selected[0]) {
        print OUT $current, " ";
        shift @selected;
      }
      $index++;
    }
  }
  close(IN);
  print OUT "\n";
          }
        }
      }
    }
  }
}
close(MEAN);

sub ManetScenario()
{
  my($count) = 0;
  my($i) = $_[$count]; $count++;

  # These parameters should be set the same when doing comparative runs
  ############### OSPF config ##################
  $HelloInterval = 2;
  $DeadInterval = 6;
  $RxmtInterval = 7;
  $MinLSInterval = 5;
  $MinLSArrival = 1;
  ############### MANET-OSPF config ##############
  $diff_hellos = ""; #"diff_hellos";
  $AckInterval = 500;  #default 1800 msec
                       #(AckInterval + propagation delay) < PushbackInterval
                       #Range in GTNetS 0 to 0.5 sec
  $wireless_flooding = 2;

  if ($i eq "ORSPfull") {
    $wireless_flooding = 1; 
    $smart_peering = "SmartPeering";
    $unsynch_adj = "UnsynchAdj";
    $PushbackInterval = 4500;   # default 2000 msec
                                #PushbackInterval + propagation delay) <
                                #RxmtInterval/2
                                #Range in GTNetS 4.5 to 9 sec
  } elsif ($i eq "OR") {
    $wireless_flooding = 1; 
    $smart_peering = "";
    $unsynch_adj = "";
    $PushbackInterval = 4500;   # default 2000 msec
                                #PushbackInterval + propagation delay) <
                                #RxmtInterval/2
                                #Range in GTNetS 4.5 to 9 sec
  } elsif ($i eq "MDRfullfull") {
    $wireless_flooding = 2; 
    $TwoHopRefresh = 3;
    $BackupWaitInterval = 1000;  # default 2000 msec
                                #Range in GTNetS 1 to 2 sec
    $AdjConnectivity = 0;
    $LSAFullness = 4;
  } elsif ($i eq "MDRunifull") {
    $wireless_flooding = 2; 
    $TwoHopRefresh = 3;
    $BackupWaitInterval = 1000;  # default 2000 msec
                                #Range in GTNetS 1 to 2 sec
    $AdjConnectivity = 1;
    $LSAFullness = 4;
  } elsif ($i eq "MDRbimincost") {
    $wireless_flooding = 2; 
    $TwoHopRefresh = 3;
    $BackupWaitInterval = 1000;  # default 2000 msec
                                #Range in GTNetS 1 to 2 sec
    $AdjConnectivity = 2;
    $LSAFullness = 1;
  } else {
    die "This is not a valid manet_scenario\n";
  }

  if ($wireless_flooding == 0) { $wf = "broadcast"; $ac=""; $lf = "";}
  elsif ($wireless_flooding == 1) { 
    $wf = "MPR"; 
    if ($smart_peering eq "SmartPeering") { $ac = "sp" }
    elsif ($smart_peering eq "") { $ac = "full" }
    if ($unsych_adj eq "UnsynchAdj") { $lf = "min";}
    elsif ($unsych_adj eq "") { $lf = "full";}
  } elsif ($wireless_flooding == 2) { 
    $wf = "MDR"; 
    if ($AdjConnectivity == 0) { $ac = "full";}
    elsif ($AdjConnectivity == 1) { $ac = "uni";}
    elsif ($AdjConnectivity == 2) { $ac = "bi";}
    if ($LSAFullness == 0) { $lf = "min";}
    elsif ($LSAFullness == 1) { $lf = "minhop";}
    elsif ($LSAFullness == 2) { $lf = "minhop2path";}
    elsif ($LSAFullness == 3) { $lf = "mdrfull";}
    elsif ($LSAFullness == 4) { $lf = "full";}
  }
}
