// Test program for Wireless Grid object
// George F. Riley.  Georgia Tech, Fall 2002

#include <iostream>
#include <stdlib.h>

#include "simulator.h"
#include "wireless-grid-rectangular.h"
#include "wireless-grid-polar.h"
#include "ipaddr.h"
#include "rng.h"
#include "node.h"
#include "mobility-random-waypoint.h"
#include "ratetimeparse.h"
#include "application-cbr.h"
#include "validation.h"  
#include "etrace.h"

int main(int argc, char** argv)
{
  //Validation::Init(argc, argv);
  Simulator s;
  //WirelessGrid(1000, Uniform(100,1000), Uniform(0,1000));
  //WirelessGridRectangular(1000, 1000, Uniform(100,1000),
  //             Sequential(0,1000,100), Sequential(0,1000,100));
  Count_t  nNodes = 2;
	Meters_t  distance = 100, range = 100;
  if (argc > 1) distance = atof(argv[1]);
	if (argc > 2) range = atof(argv[2]);
  IPAddr_t firstIP = IPAddr("192.168.0.1");
  Location l(500,500);
  WirelessGridPolar g(l,
                      Constant(nNodes),
                      Exponential(250, 500),
                      Uniform(0.0, 360.0),
                      firstIP);
  ETrace* egs = ETrace::Instance();
  egs->Open("testwirelessmodels.tr");
  egs->LayerOff(2); //don't etrace this layer

  
  for (Count_t i = 0; i < g.Size(); ++i)
    {
      Node* n = g.GetNode(i);
      //n->SetRadioRange(l.X() / 10);
      n->SetRadioRange(range);

			// set location
			n->SetLocation(0,i*distance,0);

			//CBR
		  IPAddr_t remoteIP = (i < (g.size -1 ) ?
						        (n->GetIPAddr() + 1) : (n->GetIPAddr() - 1));

      CBRApplication* cbr = (CBRApplication*)n->AddApplication(
          CBRApplication(remoteIP, 1000, NO_PORT, 64*8, 64));
      cbr->Start(1);
			cbr->Stop(10000);
    }
  /*
  if (!Validation::noAnimation)
    {
      s.AnimateWirelessTx(true);
      s.StartAnimation(0, true);
      s.PauseAnimation(100);
      s.AnimationUpdateInterval(Time("1ms")); // 1ms initial update rate
    }
	a*/
  s.StopAt(10005);
  s.Run();
	//s.PrintStats();
	//tats::Print();
}
